from datetime import datetime, timedelta
from django.contrib.auth.hashers import make_password
import jwt
import requests
from pathlib import Path
from main.settings import BASE_DIR
from ..models import BaseInfo, Auth
from ..utils.model_handle import parse_model
from ..services.wechat_service import WechatService
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""
jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

redis_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))
redis_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))

redis_host = redis_main_config_dict.redis_host or redis_module_config_dict.redis_host or ""
redis_port = redis_main_config_dict.redis_port or redis_module_config_dict.redis_port or ""
redis_password = redis_main_config_dict.redis_password or redis_module_config_dict.redis_password or ""


class LoginRegister:
    # @staticmethod
    # def login_register(phone, code):
    #     wechat_openid = LoginRegister.get_openid(code=code)
    #     current_user = BaseInfo.objects.filter(phone=phone).filter()
    #     current_user = parse_model(current_user)
    #     if not current_user:
    #         base_info = {
    #             'username': '',
    #             'nickname': phone,
    #             'phone': phone,
    #             'email': '',
    #             'fullname': '请修改用户名',
    #             'wechat_openid': wechat_openid['openid']
    #         }
    #         BaseInfo.objects.create(**base_info)
    #         current_user = BaseInfo.objects.filter(phone=phone).filter()
    #         current_user = parse_model(current_user)
    #         current_user = current_user[0]
    #         # 生成登录token
    #         token = LoginRegister.set_token(current_user.get('id', None), phone)
    #
    #         # 创建用户登录信息，绑定token
    #         auth = {
    #             'user_id': current_user.get('id', None),
    #             'password': make_password('123456', None, 'pbkdf2_sha1'),
    #             'plaintext': '123456',
    #             'token': token,
    #         }
    #         Auth.objects.update_or_create({'user_id': current_user.get('id', None)}, **auth)
    #         auth_set = Auth.objects.filter(user_id=current_user.get('id', None), password__isnull=False).order_by(
    #             '-update_time').first()
    #         return 0, {'token': auth_set.token, 'user_info': current_user}
    #     current_user = current_user[0]
    #     auth_set = Auth.objects.filter(user_id=current_user.get('id', None), password__isnull=False).order_by(
    #         '-update_time').first()
    #     return 0, {'token': auth_set.token, 'user_info': current_user}

    @staticmethod
    def set_token(user_id, account):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=int(expire_day),
            seconds=int(expire_second)
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
            key=jwt_secret_key
        )

    @staticmethod
    def get_openid(code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': app_id,
            'secret': app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import json
from urllib.parse import parse_qs
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from rest_framework.request import Request
from ..utils.parse_json import parse_json
# import xmltodict


# 请求参数解析
def request_params_wrapper(func):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        解析request参数，适配多种body格式。
        PS :注意使用该装饰器之后必搭配*args，**kwargs须使用
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance, request, *args, request=request, request_params={}, **kwargs, )

        # 参数解析
        content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
        method = request.method
        if content_type == "text/plain" or method == "GET":  # 不指定则默认这种content-type
            try:
                body = request.body.decode("utf-8")
                data = json.loads(body)
            except Exception as e:
                # 允许get请求的query参数传json格式字符串，如：?group_list=["basics","bid-online"]
                data = parse_json(request.GET.dict())
                if not data:
                    data = request.POST
                if not data:
                    data = {}
        elif content_type == "application/json":
            data = json.loads(request.body)
        elif content_type == "multipart/form-data":
            data = request.POST
        # elif content_type == "application/xml":
        #     try:
        #         data = xmltodict.parse(request.body)
        #         data = data.get("body") or data.get("data", {})
        #     except Exception as e:
        #         data = {}
        elif content_type == "application/x-www-form-urlencoded":
            data = parse_qs(request.body.decode())
            if data:
                data = {k: v[0] for k, v in data.items()}
            else:
                data = {}
        else:
            data = getattr(request, 'data', {})

        # 闭包抛出
        kwargs.pop("request_params", None)  # 避免重复执行报错
        kwargs.pop("request", None)  # 避免重复执行报错
        return func(instance, request, *args, request=request, request_params={k: v for k, v in data.items()}, **kwargs)

    return wrapper



# encoding: utf-8
"""
@project: djangoModel->__init__
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 监听者逻辑类，分目录管理。每个目录的初始化文件对外暴露自动注册类 ===>>> 然后再服务注册脚本实现自动监听 service_register.py
@created_time: 2023/5/11 11:04
"""



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .jwa import std_hash_by_alg
from .jwk import (
    AbstractJWKBase,
    jwk_from_dict,
    jwk_from_bytes,
    jwk_from_pem,
    jwk_from_der,
    supported_key_types,
)
from .jwkset import JWKSet
from .jwa import (
    AbstractSigningAlgorithm,
    supported_signing_algorithms,
)
from .jwt import JWT


__all__ = [
    # .jwa
    'std_hash_by_alg',
    # .jwk
    'AbstractJWKBase',
    'jwk_from_bytes',
    'jwk_from_dict',
    'jwk_from_pem',
    'jwk_from_der',
    'supported_key_types',
    # .jwkset
    'JWKSet',
    # .jws
    'AbstractSigningAlgorithm',
    'supported_signing_algorithms',
    # .jwt
    'JWT',
]



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import json
from urllib.parse import parse_qs
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from rest_framework.request import Request
import functools


# import xmltodict


def x_request_wrapper(func):
    '''
    请求参数解析装饰器。
    1、基于APIView类的入参进行解析，在原参数基础上，追加 current 请求体，request_params 请求参数两个入参。
    2、将适配多种body格式。兼容 APIView与View的情况，View 没有request.data
    示例：
    原参数：def get(self, request, *args, **kwargs)
    变为：def get(self, request, request_params, *args, **kwargs)
    '''

    @functools.wraps(func)  # 更新 wrapper 函数的 __name__、__doc__、__module__ 和 __annotations__ 属性
    def wrapper(instance, request=None, *args, **kwargs):
        """
        解析request参数，适配多种body格式。兼容 APIView与View的情况，View 没有request.data
        PS :注意使用该装饰器之后必搭配*args，**kwargs须使用
        @param instance 实例是一个继承APIView基类的实例
        @param request REST封装的Request实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        # print("> x_request_wrapper: wrapper: ", instance, request, args, kwargs)
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            current = instance
        if isinstance(request, WSGIRequest) or isinstance(request, Request) or isinstance(request, ASGIRequest):
            current = request
        if current is None:
            return func(instance, *args, request=current, request_params={}, **kwargs)

        # 参数解析
        content_type = current.META.get('CONTENT_TYPE', "").split(";")[0]
        method = current.method
        if content_type == "text/plain" or method == "GET":  # 不指定则默认这种content-type
            try:
                body = current.body.decode("utf-8")
                data = json.loads(body)
            except Exception as e:
                # 允许get请求的query参数传json格式字符串，如：?group_list=["basics","bid-online"]
                data = parse_json(current.GET.dict())
                if not data:
                    data = current.POST
                if not data:
                    data = {}
        elif content_type == "application/json":
            data = json.loads(current.body)
        elif content_type == "multipart/form-data":
            data = current.POST
        # elif content_type == "application/xml":
        #     try:
        #         data = xmltodict.parse(current.body)
        #         data = data.get("body") or data.get("data", {})
        #     except Exception as e:
        #         data = {}
        elif content_type == "application/x-www-form-urlencoded":
            data = parse_qs(current.body.decode())
            if data:
                data = {k: v[0] for k, v in data.items()}
            else:
                data = {}
        else:
            data = getattr(current, 'data', {})

        # 闭包抛出
        kwargs.pop("request_params", None)  # 避免重复执行报错
        kwargs.pop("current", None)  # 避免重复执行报错

        # 克隆参数
        copy_params = {k: v for k, v in data.items()}
        return func(instance, *args, request=current, request_params=copy_params, **kwargs)

    return wrapper


# json 结果集返回
def parse_json(result):
    if result is None:
        return None

    if type(result) is str:
        try:
            result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
        except Exception as e:
            return result
    if type(result) is list:
        for i, v in enumerate(result):
            result[i] = parse_json(v)
    if type(result) is dict:
        for k, v in result.items():
            result[k] = parse_json(v)
    return result



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
import json
import sys
from logging import getLogger
from pathlib import Path

from django.forms import model_to_dict
import requests
from django.contrib.auth.hashers import make_password

from main.settings import BASE_DIR
if 'xj_role' in sys.modules:
    from xj_role.services.user_group_service import UserGroupService
from xj_user.services.login_service import LoginService
from xj_user.services.wechat_service import WechatService
from ..models import BaseInfo, Auth
from ..services.user_detail_info_service import DetailInfoService
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.nickname_generate import gen_one_word_digit
from ..utils.j_ulid_field import JULIDField

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""
jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

logger = getLogger('log')

# 用户短信服务
class UserSmsService:

    # 手机短信登录
    def phone_login(self, phone,  platform_id=None, other_params=None, nonexistent_alert=False):
        """
        :param phone:
        :param platform_id:
        :param other_params:
        :param nonexistent_alert: 如果用户不存在则报警，默认False
        :return:
        """
        if other_params is None:
            other_params = {}
        # 根据手机号获取用户
        current_user = BaseInfo.objects.filter(phone=phone).first()
        # print(">>>用户信息 current_user ", current_user)

        # if not sso_serve_id:
        #     return None, "单点登录不能为空"

        if not platform_id:
            return None, "平台不能为空"
        # 如果用户不存在且要求报警
        if not current_user and nonexistent_alert:
            return None, "平台用户不存在"

        # 如果用户未注册 用手机号进行注册
        if not current_user:
            base_info = {
                'uuid': JULIDField.get_u12(),
                'username': get_short_id(8),
                'nickname': gen_one_word_digit(),
                'phone': phone,
                'email': '',
            }

            BaseInfo.objects.create(**base_info)

            # 注册完成后 重新获取用户信息
            user_info_set = BaseInfo.objects.filter(phone=phone).first()
            user_info = model_to_dict(user_info_set)
            # platforms_to_users解包
            user_info['platforms_to_users'] = [model_to_dict(platforms) for platforms in
                                               user_info['platforms_to_users']]
            # 检验单点登录信息
            # sso_set, sso_err = LoginService.sso_verify(sso_serve_id, user_info.get('id', None), sub_appid)
            # if sso_err:
            #     return None, sso_err

            password = str(other_params.get("password", "123456"))
            # 生成登录token
            token = LoginService.make_token(user_info.get("id"), user_info.get('uuid', None), phone, platform_id)
            # 创建用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_info.get('id', None), token, password=password)
            if auth_err:
                return None, auth_err
            return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None
        # 如果用户已经存在
        else:
            # 存在当前用户，直接进行授权
            user_info = model_to_dict(current_user)
            # platforms_to_users解包
            user_info['platforms_to_users'] = [model_to_dict(platforms) for platforms in  user_info['platforms_to_users'] ]

            # 检验单点登录信息
            # sso_set, sso_err = LoginService.sso_verify(sso_serve_id, user_info.get('id', None), sub_appid)
            # if sso_err:
            #     return None, sso_err

            # 创建用户登录信息，绑定token
            token = LoginService.make_token(user_info.get("id"), user_info.get('uuid', None), user_info.get('phone', None), platform_id)
            # 修改用户登录信息，绑定token
            auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', None), token=token,
                                                         is_create=False)
            if auth_err:
                return None, auth_err


        return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None



from django.test import TestCase
from ..services.extend_field_service import ExtendFieldService
from ..models import ExtendField

class TestExtendFieldService(TestCase):
    def setUp(self):
        # 初始化测试数据
        self.test_params = {
            "field_index": "field_1",
            "field": "test_field",
            "label": "Test Label",
            "type": "text",
            "unit": "unit",
            "default": "default_value",
            "config": "{}",
            "sort": 1,
            "description": "Test Description"
        }
        # 删除可能存在的旧数据
        ExtendField.objects.filter(field_index=self.test_params["field_index"]).delete()
        self.test_extend_field = ExtendField.objects.create(**self.test_params)

    def test_extend_field_list_with_params(self):
        # 测试带参数的 extend_field_list 方法
        result, err = ExtendFieldService.extend_field_list(params=self.test_params)
        print("> TestExtendFieldService: test_extend_field_list_with_params:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertGreater(len(result['list']), 0)

    def test_extend_field_list_without_params(self):
        # 测试不带参数的 extend_field_list 方法
        result, err = ExtendFieldService.extend_field_list()
        print("> TestExtendFieldService: test_extend_field_list_without_params:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertGreater(len(result['list']), 0)

    def test_extend_field_add(self):
        # 测试 extend_field_add 方法
        new_field_params = {
            "field_index": "field_4",  # 确保这个值是唯一的
            "field": "new_field",
            "label": "New Label",
            "type": "number",
            "unit": "new_unit",
            "default": "new_default",
            "config": "{}",
            "sort": 2,
            "description": "New Description"
        }
        result, err = ExtendFieldService.extend_field_add(new_field_params)
        print("> TestExtendFieldService: test_extend_field_add:", result, err)
        self.assertEqual(err, None)
        self.assertIsNotNone(result)
        self.assertEqual(result['field_index'], new_field_params['field_index'])

    def test_extend_field_add_with_duplicate_field_index(self):
        # 测试 extend_field_add 方法，使用重复的 field_index
        duplicate_field_params = {
            "field_index": "field_1",  # 重复的 field_index
            "field": "duplicate_field",
            "label": "Duplicate Label",
            "type": "text",
            "unit": "unit",
            "default": "default_value",
            "config": "{}",
            "sort": 3,
            "description": "Duplicate Description"
        }
        result, err = ExtendFieldService.extend_field_add(duplicate_field_params)
        print("> TestExtendFieldService: test_extend_field_add_with_duplicate_field_index:", result, err)
        self.assertIsNotNone(err)
        self.assertEqual(err, "field_index 已存在")

    def test_extend_field_edit(self):
        # 测试 extend_field_edit 方法
        edit_params = {
            "label": "Updated Label",
            "description": "Updated Description"
        }
        result, err = ExtendFieldService.extend_field_edit(edit_params, self.test_extend_field.id)
        print("> TestExtendFieldService: test_extend_field_edit:", result, err)
        self.assertEqual(err, None)
        self.assertIsNotNone(result)
        self.assertEqual(result['label'], edit_params['label'])
        self.assertEqual(result['description'], edit_params['description'])

    def test_extend_field_delete(self):
        # 测试 extend_field_delete 方法
        # 删除可能存在的旧数据
        ExtendField.objects.filter(field_index=self.test_params["field_index"]).delete()
        self.test_extend_field = ExtendField.objects.create(**self.test_params)


from django.test import TestCase
from ..services.user_service import UserService

class TestUserAdd(TestCase):
    def setUp(self):
        # 初始化测试数据
        self.test_params = {
            "account": "testuser",
            "password": "testpassword",
            "fullname": "Test User",
            "nickname": "testnick",
            "user_type": "PERSON",
            "phone": "1234567890",
            "email": "test@example.com"
        }

    def test_user_add_with_valid_params(self):
        # 测试带有效参数的 user_add 方法
        # 确保手机号格式正确
        self.test_params["phone"] = "13800138000"  # 使用有效的手机号格式
        result, err = UserService.user_add(params=self.test_params)
        print("> TestUserAdd: test_user_add_with_valid_params:", result, err)
        self.assertEqual(err, None)
        self.assertIn('id', result)
        self.assertIn('username', result)
        self.assertIn('fullname', result)
        self.assertIn('nickname', result)
        self.assertIn('phone', result)
        self.assertIn('email', result)

    def test_user_add_with_missing_account(self):
        # 测试缺少 account 参数的 user_add 方法
        invalid_params = self.test_params.copy()
        invalid_params.pop("account")
        result, err = UserService.user_add(params=invalid_params)
        print("> TestUserAdd: test_user_add_with_missing_account:", result, err)
        self.assertNotEqual(err, None)
        self.assertEqual(result, None)

    def test_user_add_with_invalid_phone(self):
        # 测试带无效手机号格式的 user_add 方法
        invalid_params = self.test_params.copy()
        invalid_params["phone"] = "invalid_phone"
        result, err = UserService.user_add(params=invalid_params)
        print("> TestUserAdd: test_user_add_with_invalid_phone:", result, err)
        self.assertNotEqual(err, None)
        self.assertEqual(result, None)

    def test_user_add_with_invalid_email(self):
        # 测试带无效邮箱格式的 user_add 方法
        invalid_params = self.test_params.copy()
        invalid_params["email"] = "invalid_email"
        result, err = UserService.user_add(params=invalid_params)
        print("> TestUserAdd: test_user_add_with_invalid_email:", result, err)
        self.assertNotEqual(err, None)
        self.assertEqual(result, None)

    def test_user_add_with_existing_account(self):
        # 测试带已存在账号的 user_add 方法
        # 先创建一个用户
        UserService.user_add(params=self.test_params)
        # 再次尝试创建相同账号的用户
        result, err = UserService.user_add(params=self.test_params)
        print("> TestUserAdd: test_user_add_with_existing_account:", result, err)
        self.assertNotEqual(err, None)
        self.assertEqual(result, None)

    def test_user_add_with_default_values(self):
        # 测试带默认值的 user_add 方法
        # 只提供必填参数，其他参数使用默认值
        minimal_params = {
            "account": "testuser2",
            "password": "testpassword"
        }
        result, err = UserService.user_add(params=minimal_params)
        print("> TestUserAdd: test_user_add_with_default_values:", result, err)
        self.assertEqual(err, None)
        self.assertIn('id', result)
        self.assertIn('username', result)
        self.assertIn('fullname', result)
        self.assertIn('nickname', result)
        self.assertIn('phone', result)
        self.assertIn('email', result)


"""
Created on 2023-07-20
@author:高栋天
@description:自定义方法
"""
from ..libs import pinyin
import uuid
import json
import sys
import time
import os
import string
import random
# import pytz
import datetime
from django.utils import timezone
# from dateutil.parser import parse
# from dateutil.tz import tzlocal
# import pandas as pd
from datetime import datetime
from decimal import Decimal


# # 获取当前时间
# def get_current_time():
#     # TODO USE_TZ = False 时会报错 如果USE_TZ设置为True时，Django会使用系统默认设置的时区，即America/Chicago，此时的TIME_ZONE不管有没有设置都不起作用。
#     tz = pytz.timezone('Asia/Shanghai')
#     # 返回datetime格式的时间
#     now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S.%f")
#     # now = datetime.strptime(now_time, '%Y-%m-%d %H:%M:%S')
#     now = datetime.now().strptime(now_time, "%Y-%m-%d %H:%M:%S.%f")
#     return now


# 时间精确到微秒
def append_microsecond_to_datetime(datetime_str: str) -> datetime:
    """
    将当前微秒追加到日期时间字符串中。如果字符串仅包含日期，当前时间也附加在微秒之前。
    """
    # Check if the datetime string contains a time
    if len(datetime_str.split(' ')) == 2:
        current_microsecond = datetime.now().strftime("%f")
        datetime_str = f'{datetime_str}.{current_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    else:
        # Get the current time and microsecond
        current_time_microsecond = datetime.now().strftime("%H:%M:%S.%f")
        datetime_str = f'{datetime_str} {current_time_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    return dt


# 数据key替换
def replace_dict_key(dictionary, old_key, new_key):
    """
       :param dictionary: 字典
       :param old_key: 旧key
       :param new_key: 替换的新key
   """
    if old_key in dictionary:
        dictionary[new_key] = dictionary.pop(old_key)
    return dictionary


# 数据字典key替换
def replace_key_in_dict(original_dict, old_key, new_key):
    """
       :param original_dict: 字典
       :param old_key: 旧key
       :param new_key: 替换的新key
   """
    copied_dict = original_dict.copy()
    if old_key in copied_dict:
        copied_dict[new_key] = copied_dict.pop(old_key)
    return copied_dict


# 数据列表key替换
def replace_key_in_list_dicts(list_dicts, old_keys, new_key):
    """
       :param list_dicts: 列表
       :param old_key: 旧key
       :param new_key: 替换的新key
    """
    for d in list_dicts:
        for old_key in old_keys:
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 字典替换字典
def replace_key_in_dict_replacement_dicts(dict_data, replacement_dict):
    """
      :param dict_data: 字典
      :param replacement_dict: 替换的字典映射
   """
    for old_key, new_key in replacement_dict.items():
        if old_key in dict_data:
            dict_data[new_key] = dict_data.pop(old_key, dict_data[old_key])
    return dict_data


# 数据列表字典key替换
def replace_key_in_list_replacement_dicts(list_dicts, replacement_dict):
    """
       :param list_dicts: 列表
       :param replacement_dict: 要替换的字典
    """
    for d in list_dicts:
        for old_key, new_key in replacement_dict.items():
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 字符串转列表
def parse_integers(value):
    if isinstance(value, str):
        if "," in value:
            lst = [int(num) for num in value.split(",")]
        else:
            lst = [int(value)]
    elif isinstance(value, int):
        lst = [value]
    else:
        raise TypeError("不支持的值类型。应为字符串或int")

    return lst


# 保留两位小数
def keep_two_decimal_places(str_num):
    """
    :param str_num: 要处理的字段
    """
    result_num = format(float(str_num), "")

    if len(result_num.split(".")[-1]) < 2:
        result_num = result_num + "0"
    return result_num


# 生成一个长度为16的密码
def generate_password(length=16):
    """
     :param length: 长度
     """
    # 合并所有可能的字符，包括大小写字母、数字和标点符号
    # all_chars = string.ascii_letters + string.digits + string.punctuation
    all_chars = string.ascii_letters + string.digits
    # length = random.randint(8, 12)
    # 随机选择指定数量的字符
    password = ''.join(random.choice(all_chars) for _ in range(length))

    return password


# 数字表示生成几位, True表示生成带有字母的 False不带字母的
def get_code(n=6, alpha=False):
    """
     :param n: 要生成的位数
     :param alpha: True表示生成带有字母的 False不带字母的
     """
    s = ''  # 创建字符串变量,存储生成的验证码
    for i in range(n):  # 通过for循环控制验证码位数
        num = random.randint(1, 9)  # 生成随机数字0-9
        if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
            upper_alpha = chr(random.randint(65, 90))
            lower_alpha = chr(random.randint(97, 122))
            num = random.choice([num, upper_alpha, lower_alpha])
        s = s + str(num)
    return s


# 检查列表字段是否存在
def find(list, keyword):
    """
   :param list: 列表
   :param keyword: 要检查的字段
   """
    try:
        list.index(keyword)
        return True
    except ValueError:
        return False


# # 批量数据时间格式化
# def format_dates(data, date_fields):
#     """
#    :param data: 列表
#    :param date_fields: 要格式化的字段列表
#    """
#     for item in data:
#         for field in date_fields:
#             if field in item and item[field]:
#                 try:
#                     # 如果字段已经是 datetime 对象，就无需解析
#                     if isinstance(item[field], datetime):
#                         date = item[field]
#                     else:
#                         # 尝试解析并格式化日期
#                         date = parse(item[field])
#                     # 使用 strftime 格式化日期
#                     item[field] = date.astimezone(tzlocal()).strftime('%Y-%m-%d %H:%M:%S')
#                 except ValueError:
#                     # 如果解析失败，保留原来的值
#                     pass
#     return data


# 列表根据指定key计算value求和
# def aggregate_data(data_list, group_field, agg_fields):
#     """
#    :param data_list: 列表
#    :param group_field: 分组规则
#    :param agg_fields: 要求和的字段
#    """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field])
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field)[agg_fields].sum().reset_index()
#
#     # 将结果转换回字典列表
#     grouped_data = grouped.to_dict('records')
#
#     return grouped_data

# def aggregate_data(data_list, group_field, agg_fields):
#     """
#     :param data_list: 列表
#     :param group_field: 分组规则
#     :param agg_fields: 要求和的字段列表
#     """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值
#
#     # 将NaN值替换为0
#     df.fillna(0, inplace=True)
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field)[agg_fields].sum().reset_index()
#
#     # 将结果转换回字典列表
#     grouped_data = grouped.to_dict('records')
#
#     return grouped_data

# def aggregate_data(data_list, group_field, agg_fields):
#     """
#     :param data_list: 列表
#     :param group_field: 分组规则
#     :param agg_fields: 要求和的字段列表
#     """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值
#
#     # 将NaN值替换为0
#     df.fillna(0, inplace=True)
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field, as_index=False)[agg_fields].sum()
#
#     return grouped.to_dict('records')

# def aggregate_data(data_list, group_field, agg_fields, agg_result_field):
#     """
#     :param data_list: 列表
#     :param group_field: 分组规则
#     :param agg_fields: 要求和的字段列表
#     """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值
#
#     # 将NaN值替换为0
#     df.fillna(0, inplace=True)
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     df[agg_result_field] = df.groupby(group_field)[agg_fields].transform('sum')
#
#     # 选择需要的字段
#     grouped_data = df.drop(agg_fields, axis=1).drop_duplicates(subset=group_field).to_dict('records')
#
#     return grouped_data


# json模板替换
def replace_placeholders(data, replacements):
    """
   :param data: 要替换的数据
   :param replacements: 要替换的字典
   """
    # 将 data 转换为 JSON 格式的字符串
    data_str = str(data)

    # 依次替换每一个 "{{}}"
    for replacement in replacements:
        data_str = data_str.replace("{{}}", replacement, 1)

    # 将字符串重新转换为字典
    data = eval(data_str)

    return data


# 获取程序运行时间
def testRunTime():
    start = datetime.now()
    for i in range(1000):
        for j in range(500):
            m = i + j
            print(m)
    end = datetime.now()
    print(end - start)
    return end - start


# 将dict转换为json
def convert_dict_to_json(data):
    """
     :param data: 要转换的字典
     """

    def convert_values_to_str(value):
        if isinstance(value, dict):
            return {convert_values_to_str(key): convert_values_to_str(val) for key, val in value.items()}
        elif isinstance(value, list):
            return [convert_values_to_str(val) for val in value]
        elif isinstance(value, Decimal):
            return str(value)
        elif value is None:
            return value
        else:
            return str(value)

    converted_data = convert_values_to_str(data)
    json_str = json.dumps(converted_data, ensure_ascii=False)
    return json_str


# for 列表搜索
def search_in_dict_list(dict_list, query_dict):
    """
     :param dict_list: 要过滤的列表
     :param query_dict: 要筛选的字典
     """
    return [d for d in dict_list if all(d.detail(k) == v for k, v in query_dict.items())]


# # Pandas库 列表搜索(支持模糊搜索)
# def search_in_dict_list(dict_list, query_dict):
#     """
#    :param dict_list: 要过滤的列表
#    :param query_dict: 要筛选的字典
#    """
#     df = pd.DataFrame(dict_list)
#     original_length = len(df)
#
#     for key, value in query_dict.items():
#         if isinstance(value, str):
#             df = df[df[key].str.contains(value)]
#         else:
#             df = df[df[key] == value]
#
#     not_matching_count = original_length - len(df)
#     return df.to_dict('records'), not_matching_count


# 字典过滤
def filter_dict_by_keys(dict_obj, keys):
    """
    :param dict_obj: 要过滤的字典
    :param keys: 要筛选的字段列表
    """
    keys_set = set(keys)  # Convert list to set
    return {k: v for k, v in dict_obj.items() if k in keys_set}


# # 列表字段提取
# def extract_values(dict_list, key):
#     """
#     :param dict_list: 要提取的列表
#     :param key: 需要提前的key
#     """
#     if not dict_list:  # 如果dict_list为空，则返回空列表
#         return []
#     df = pd.DataFrame(dict_list)
#     return df[key].tolist()


# 替换sql里面的占位符
def generate_query_string(query_string, param_list):
    """
      :param query_string: 要替换的字符串
      :param param_list: 替换的列表
      """
    # 替换问号
    for param in param_list:
        query_string = query_string.replace('?', str(param), 1)
    return query_string


# 根据支付宝微信规则生成28位不重复的交易号
def generate_trade_no():
    # 生成不重复的UUID
    trade_no = str(uuid.uuid4().int)
    # 判断交易号长度，不足补0
    if len(trade_no) < 28:
        trade_no = trade_no.zfill(28)
    # 根据支付宝和微信的规则进行格式化
    # trade_no = trade_no[:4] + '-' + trade_no[4:8] + '-' + trade_no[8:12] + '-' + trade_no[12:20] + '-' + trade_no[20:]
    trade_no = trade_no[:4] + trade_no[4:8] + trade_no[8:12] + trade_no[12:20] + trade_no[20:]

    if len(trade_no) < 28:
        trade_no = trade_no.zfill(28)
    return trade_no


# 根据中文生成大写缩写
def get_chinese_initials(chinese):
    """
      :param chinese: 中文
    """
    chinese_pinyin = pinyin.get(chinese, format='strip', delimiter=' ')
    initials = [p[0].upper() for p in chinese_pinyin.split()]
    return ''.join(initials)



import argparse
from pinyin import get

from pinyin._compat import u


def pinyin():
    parser = argparse.ArgumentParser()
    parser.add_argument("chars", help="Input chinese words")
    args = parser.parse_args()

    if not args.chars:
        parser.print_help()
        return

    print(get(u(args.chars)))



import datetime
import uuid

from django.db import models
from django.utils import timezone
from .utils.j_django_simple_api import JDjangoSimpleApi
from .utils.j_ulid_field import JULIDField


models.Model.to_json = JDjangoSimpleApi.serialize_model

prefix = ''  # 'bx' + '_'

user_vip_choices = [(1, '普通用户'), (2, '高级会员'), (3, '超级会员'), (4, '年费会员'), ]
user_type_choices = [('PERSON', '个人用户'), ('COMPANY', '企业用户'), ('COMPANYADMIN', '企业管理员'), ('ADMIN', '管理员用户')]

class BaseInfo(models.Model):
    """ 1、User_Base_Info 基础信息表 [NF1] """
    id = models.AutoField(verbose_name='ID', primary_key=True)
    uuid = JULIDField(verbose_name='UUID', default=JULIDField.get_u12(), unique=True, editable=True, blank=True, null=True,
                      db_index=True,)
    user_no = models.CharField(verbose_name='用户编号', max_length=32, unique=True, db_index=True, blank=True, null=True)
    username = models.CharField(verbose_name='用户名', max_length=32, unique=True, blank=True, null=True, db_index=True,
                                help_text='登录的用户名')
    nickname = models.CharField(verbose_name='昵称', max_length=128, blank=True, null=True, help_text='')
    fullname = models.CharField(verbose_name='姓名', max_length=128, blank=True, null=True, db_index=True,
                                help_text='用户的姓名（未实名验证）')
    phone = models.CharField(verbose_name='手机', max_length=16, unique=False, blank=True, null=True, db_index=True,
                             help_text='登录绑定手机号')
    email = models.EmailField(verbose_name='邮箱', unique=False, blank=True, null=True, db_index=True, help_text='登录绑定邮箱')
    avatar = models.CharField(verbose_name='个人头像', max_length=255, blank=True, null=True, help_text='')
    user_type = models.CharField(verbose_name='用户类型', max_length=32, default='PERSON', db_index=True,choices=user_type_choices,
                                 blank=True, null=True, help_text='PERSON 个人用户, COMPANY 企业用户, ADMIN 管理员用户')
    user_vip = models.IntegerField(verbose_name='用户会员', default=1, db_index=True, blank=True, null=False,
                                        choices=user_vip_choices, help_text='用户会员权限')
    user_level = models.IntegerField(verbose_name='用户等级', default=1, db_index=True, blank=True, null=False, help_text='')
    user_info = models.JSONField(verbose_name='用户信息', blank=True, null=True, help_text='用户扩展信息')
    register_time = models.DateTimeField(verbose_name='注册时间', default=timezone.now, blank=True, null=True, help_text='')
    register_ip = models.GenericIPAddressField(verbose_name='注册IP', protocol='both', blank=True, null=True, help_text='')
    privacies = models.JSONField(verbose_name='用户隐私设置', blank=True, null=True, help_text='用户隐私设置')
    platforms_to_users = models.ManyToManyField(to='Platform', through="PlatformsToUsers",
                                                through_fields=("user", "platform"))
    is_auth = models.BooleanField(verbose_name='是否实名认证', default=False, blank=True, null=True, )
    is_delete = models.BooleanField(verbose_name='是否删除', default=False, blank=True, null=True, )
    is_using = models.BooleanField(verbose_name='是否使用', default=True, blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_base_info'
        # unique_together = [['platform', 'platform_uid'], ['platform', 'username']]
        verbose_name_plural = "01. 用户 - 基础信息"
        # ordering = ["-register_time"]
        # app_label = 'xj_user'

    def __str__(self):
        # return f"{self.username}({self.fullname})"
        return f"{self.fullname} ({self.username})"

    def user_info_short(self):
        if len(str(self.user_info)) > 30:
            return '{}...'.format(str(self.user_info)[0:30])
        return self.user_info

    user_info_short.short_description = '用户信息'

    def privacies_short(self):
        if len(str(self.privacies)) > 30:
            return '{}...'.format(str(self.privacies)[0:30])
        return self.privacies

    privacies_short.short_description = '用户隐私设置'

    def uuid_short(self):
        return '{}'.format(str(self.uuid)[0:8])

    uuid_short.short_description = 'UUID'


id_card_type_choices = [
    ('IDENTITY', '身份证'),
    ('PASSPORT', '护照'),
    ('DRIVING', '驾驶证'),
    ('BUSINESS', '营业执照'),
]
language_choices = [
    ('cn', '简体中文'),
    ('en', 'English'),
    ('ru', 'Русский'),
    ('tc', '繁体中文'),
    ('fr', 'Français'),
    ('de', 'Deutsch'),
    ('jp', '日本語'),
]


class DetailInfo(models.Model):
    sex_choices = [("0", "未知"), ("1", "男"), ("2", "女"), ]
    """ 2、User_Detail_Info 详细信息表 [1-1] """
    user = models.OneToOneField(BaseInfo, verbose_name='用户', unique=True, db_column="user_id", db_constraint=False,
                                on_delete=models.DO_NOTHING, related_name='user_detail_info', help_text='登录用的用户名')
    # user_uuid = models.UUIDField(verbose_name='用户UUID', default=uuid.uuid4, unique=True, blank=True, null=True)  # 对同一数据库内并不需要这个字段，所以外键是有ID就够了 20230822 by Sieyoo
    real_name = models.CharField(verbose_name='实名姓名', max_length=50, blank=True, null=True, db_index=True,
                                 help_text='实名认证后的真名')
    sex = models.CharField(verbose_name='性别', max_length=1, blank=True, null=True, help_text='0未知 1男 2女',
                           choices=sex_choices)
    birth = models.DateField(verbose_name='生日', blank=True, null=True, help_text='')
    tags = models.CharField(verbose_name='个人标签', max_length=255, blank=True, null=True,
                            help_text='多个标签以英文;号为分隔符')
    signature = models.CharField(verbose_name='个性签名', max_length=255, blank=True, null=True, help_text='')
    avatar = models.CharField(verbose_name='个人头像', max_length=2000, blank=True, null=True, help_text='')  # TODO 待删
    cover = models.CharField(verbose_name='个人主页封面', max_length=2000, blank=True, null=True, help_text='')
    id_card_type = models.CharField(verbose_name='证件类型', max_length=32, blank=True, null=True,
                                    choices=id_card_type_choices, help_text='用于实名认证姓名时的证件类型')
    id_card_no = models.CharField(verbose_name='证件号码', max_length=32, blank=True, null=True, help_text='')
    language = models.CharField(verbose_name='我使用的语言', max_length=2, blank=True, null=True, choices=language_choices,
                                help_text='')
    region_code = models.BigIntegerField(verbose_name='所在地行政区划编码', blank=True, null=True, help_text='')
    # subcompany = models.OneToOneField('UserSubCompany', verbose_name='分子公司', unique=True, db_column="subcompany_id", db_constraint=False, on_delete=models.DO_NOTHING, help_text='分子公司ID')
    more = models.JSONField(verbose_name='更多信息', blank=True, null=True,
                            help_text='更多信息用来存放用户可能填写的扩展内容，由于很多信息不是必填或必须存在的，因此不单独建字段。')
    field_1 = models.CharField(verbose_name='字段1', max_length=255, blank=True, null=True, help_text='')
    field_2 = models.CharField(verbose_name='字段2', max_length=255, blank=True, null=True, help_text='')
    field_3 = models.CharField(verbose_name='字段3', max_length=255, blank=True, null=True, help_text='')
    field_4 = models.CharField(verbose_name='字段4', max_length=255, blank=True, null=True, help_text='')
    field_5 = models.CharField(verbose_name='字段5', max_length=255, blank=True, null=True, help_text='')
    field_6 = models.CharField(verbose_name='字段6', max_length=255, blank=True, null=True, help_text='')
    field_7 = models.CharField(verbose_name='字段7', max_length=255, blank=True, null=True, help_text='')
    field_8 = models.CharField(verbose_name='字段8', max_length=255, blank=True, null=True, help_text='')
    field_9 = models.CharField(verbose_name='字段9', max_length=255, blank=True, null=True, help_text='')
    field_10 = models.CharField(verbose_name='字段10', max_length=255, blank=True, null=True, help_text='')
    field_11 = models.TextField(verbose_name='字段11', blank=True, null=True, help_text='')
    field_12 = models.TextField(verbose_name='字段12', blank=True, null=True, help_text='')
    field_13 = models.TextField(verbose_name='字段13', blank=True, null=True, help_text='')
    field_14 = models.TextField(verbose_name='字段14', blank=True, null=True, help_text='')
    field_15 = models.TextField(verbose_name='字段15', blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_detail_info'
        verbose_name_plural = "02. 用户 - 详细信息"

    def __str__(self):
        # return f"{self.username}({self.real_name})"
        return f"{self.user.username}"

    def user_short(self):
        return self.user if self.user else '用户不存在'

    user_short.short_description = '用户信息'

    # def user_uuid_short(self):
    #     return '{}'.format(str(self.user_uuid)[0:8])
    # user_uuid_short.short_description = '用户UUID'


class Auth(models.Model):
    """ 3、User_Auth 安全认证表 [1-1] """
    platform = models.ForeignKey(to='Platform', verbose_name='平台', blank=True, null=True, db_constraint=False,
                                 on_delete=models.DO_NOTHING)
    user = models.ForeignKey(BaseInfo, verbose_name='用户', db_constraint=False, on_delete=models.DO_NOTHING)
    # user_uuid = models.UUIDField(verbose_name='用户UUID', default=uuid.uuid4, unique=True, editable=True, blank=True, null=True)
    password = models.CharField(verbose_name='密码', max_length=255, blank=True, null=True, help_text='登录的密码')
    plaintext = models.CharField(verbose_name='PT', max_length=255, blank=True, null=True, help_text='')
    salt = models.CharField(verbose_name='盐', max_length=32, blank=True, null=True,
                            help_text='加密盐由六位数字和字母组成，区分大小写')
    algorithm = models.CharField(verbose_name='加密算法', max_length=16, blank=True, null=True,
                                 choices=[('DEFAULT', 'Default'), ('DISCUZ', 'Discuz (md5(md5(pwd).salt))'),
                                          ('SPSS', 'Spss (sha1(rsa(cip).salt))')],
                                 help_text='系统同时支持多种加密算法，旧版为MD5，新版为SHA1')
    token = models.CharField(verbose_name='临时令牌', max_length=255, blank=True, null=True,
                             help_text='用户令牌，用于验证用户有效期')
    ticket = models.CharField(verbose_name='临时票据', max_length=255, blank=True, null=True,
                              help_text='用户提供第三方单点登录的票据')
    last_update_ip = models.GenericIPAddressField(verbose_name='最后登录网络地址', protocol='both', blank=True, null=True)
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, help_text='')
    update_time = models.DateTimeField(verbose_name='最后登录时间', auto_now=True, blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_auth'
        verbose_name_plural = "03. 用户 - 安全认证"

    def __str__(self):
        return f"{self.user.username}"

    # def user_uuid_short(self):
    #     return '{}'.format(str(self.user_uuid)[0:8])
    # user_uuid_short.short_description = '用户UUID'

    def token_short(self):
        return '{}...{}({})'.format(str(self.token)[0:4], str(self.token)[-5:-1],
                                    len(self.token)) if self.token else self.token

    token_short.short_description = '临时票据'

    def ticket_short(self):
        return '{}...{}({})'.format(str(self.ticket)[0:4], str(self.ticket)[-5:-1],
                                    len(self.ticket)) if self.ticket else self.ticket

    ticket_short.short_description = '临时票据'


# 扩展字段的下拉选项
type_choices = [
    # 存储类型
    ("bool", "布尔型-bool"),
    ("int", "整型-int"),
    ("float", "浮点型-float"),
    ('number', '数字类型-number'),
    ("string", "字符串型-string"),
    ('text', '长文本型-text'),

    # 表单类型
    ('plain', '普通文字-plain'),
    ('input', '输入框-plain'),
    ('password', '密码框-password'),
    ('textarea', '多行文本框-textarea'),
    ('editor', '富文本编辑器-editor'),
    ('switch', '开关切换-switch'),
    ("select", "选择框-select"),
    ("radio", "单选框-radio"),
    ("checkbox", "多选框-checkbox"),
    ('cascader', '级联选择器-cascader'),
    ("color", "色彩选择器-color"),
    ('slot', '插槽-slot'),

    # 文档类型
    ('image', '图片-image'),
    ('audio', '音频-audio'),
    ('video', '视频-video'),
    ('file', '文件-file'),
    ('upload', '上传类型-upload'),

    # 时间类型
    ('time', ' 时间-time'),
    ("datetime", "日期时间-datetime"),
    ('date', '日期-date'),
    ('month', '月份-month'),
    ('year', '年-year'),
]
field_index_choices = [
    ('field_1', '自定义字段1'),
    ('field_2', '自定义字段2'),
    ('field_3', '自定义字段3'),
    ('field_4', '自定义字段4'),
    ('field_5', '自定义字段5'),
    ('field_6', '自定义字段6'),
    ('field_7', '自定义字段7'),
    ('field_8', '自定义字段8'),
    ('field_9', '自定义字段9'),
    ('field_10', '自定义字段10'),
    ('field_11', '自定义字段11'),
    ('field_12', '自定义字段12'),
    ('field_13', '自定义字段13'),
    ('field_14', '自定义字段14'),
    ('field_15', '自定义字段15'),
]


class ExtendField(models.Model):
    """ 4、User_Extend_Field 扩展字段表 [1-1] """
    field_index = models.CharField(verbose_name='映射索引名', max_length=8, unique=True, blank=True, null=True,
                                   choices=field_index_choices, help_text='映射到扩展数据表的字段名，如：field_x')
    field = models.CharField(verbose_name='自定义字段', max_length=32, unique=True, blank=True, null=True,
                             help_text='当已有字段不能满足的时候的扩展字段')
    label = models.CharField(verbose_name='标签', max_length=32, unique=True, blank=True, null=True,
                             help_text='自定义字段的标签，用于前端显示')
    type = models.CharField(verbose_name='类型', max_length=32, blank=True, null=True, choices=type_choices,
                            help_text='自定义字段的数据类型，必须和前端相匹配')
    unit = models.CharField(verbose_name='单位', max_length=32, blank=True, null=True,
                            help_text='自定义字段的显示单位，空值表示不需要配置单位')
    default = models.CharField(verbose_name='默认值', max_length=255, blank=True, null=True,
                               help_text='自定义字段的默认值，当前端用户不配置字段值时，使用该默认值')
    config = models.JSONField(verbose_name='配置', blank=True, null=True, help_text='用于前端自由配置一些属性，如选择列表')
    sort = models.IntegerField(verbose_name='排序', blank=True, null=True, help_text='')
    description = models.CharField(verbose_name='字段描述', max_length=255, blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_extend_field'
        verbose_name_plural = "04. 用户 - 扩展字段"

    def __str__(self):
        return f"{self.field}"


# class AccessLog(models.Model):
#     """ 5、*User_Access_Log 访问日志表 [1-N] """
#     user = models.ForeignKey(BaseInfo, verbose_name='用户', db_constraint=False, on_delete=models.DO_NOTHING,
#                              help_text='用户ID')
#     ip = models.GenericIPAddressField(verbose_name='IP', blank=True, null=True, help_text='登录的IP地址')
#     create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, help_text='')
#     client_info = models.JSONField(verbose_name='客户端信息', blank=True, null=True, help_text='')
#     more = models.JSONField(verbose_name='更多信息', max_length=255, blank=True, null=True, help_text='留用')
#
#     class Meta:
#         db_table = prefix + 'user_access_log'
#         verbose_name_plural = "05. 用户 - 访问日志表"
#
#     def __str__(self):
#         # return f"{self.username}({self.fullname})"
#         return f"{self.user.username}"


# class History(models.Model):
#     """ 6、*User_History 操作历史表 [1-N] """
#     user = models.ForeignKey(BaseInfo, verbose_name='用户', db_constraint=False, on_delete=models.DO_NOTHING,
#                              help_text='用户ID')
#     field = models.CharField(verbose_name='操作字段', max_length=32, blank=True, null=True,
#                              help_text='被用户修改的字段')
#     old_value = models.CharField(verbose_name='旧值', max_length=255, blank=True, null=True, help_text='修改前的值')
#     new_value = models.CharField(verbose_name='新值', max_length=255, blank=True, null=True, help_text='修改后的值')
#     create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, help_text='')
#
#     class Meta:
#         db_table = prefix + 'user_history'
#         verbose_name_plural = "06. 用户 - 操作历史表"
#
#     def __str__(self):
#         # return f"{self.username}({self.fullname})"
#         return f"{self.user.username}"


# class RestrictRegion(models.Model):
#     """ 7、*User_Restrict_Region 限制范围表"""
#     user = models.ForeignKey(BaseInfo, verbose_name='用户', db_constraint=False, on_delete=models.DO_NOTHING,
#                              help_text='用户ID')
#     region_code = models.BigIntegerField(verbose_name='允许的行政区划编码', blank=True, null=True, db_index=True,
#                                          help_text='在国内参考 GB/T2260-2002中国行政区划代码，有6位数 9位数 12位数，非中国地区以9开头')
#
#     class Meta:
#         db_table = prefix + 'user_restrict_region'
#         verbose_name_plural = "07. 用户 - 限制范围表"
#
#     def __str__(self):
#         # return f"{self.username}({self.fullname})"
#         return f"{self.user.username}"


class Platform(models.Model):
    """ 8、*User_Platform 平台表 """
    id = models.IntegerField(verbose_name='ID', primary_key=True, help_text='必填。自动生成。')
    platform_id = models.IntegerField(verbose_name='平台ID', primary_key=False, unique=True,
                                      help_text='必填。不自动生成，由运营人员统一设置。')
    platform_code = models.CharField(verbose_name='平台代码', max_length=16, unique=True, help_text='必填。平台唯一识别码')
    platform_name = models.CharField(verbose_name='平台名称', max_length=32,
                                     help_text='必填。平台名称可以是中文、英文、俄文等。')
    root_category_value = models.CharField(verbose_name='根类别值', max_length=32, help_text='唯一值，信息模块根类别值')

    class Meta:
        db_table = prefix + 'user_platform'
        verbose_name_plural = "07. 用户 - 平台表"

    def __str__(self):
        return f"{self.platform_name}"

    # 以上代码执行的Sql语句
    # CREATE TABLE `user_platform`(`platform_id` integer NOT NULL PRIMARY KEY, `platform_name` varchar(128) NOT NULL);


class PlatformsToUsers(models.Model):
    """ 9、*User_Platforms_To_Users - 多对多平台记录表 [N-N] """
    user = models.ForeignKey(BaseInfo, verbose_name='用户', on_delete=models.DO_NOTHING, db_column='user_id',
                             help_text='')
    platform = models.ForeignKey(Platform, verbose_name='平台', to_field="platform_id", db_constraint=False,
                                 on_delete=models.DO_NOTHING, help_text='')
    platform_user_id = models.BigIntegerField(verbose_name='平台用户ID', blank=True, null=True, db_index=True,
                                              help_text='')

    class Meta:
        db_table = prefix + 'user_platforms_to_users'
        verbose_name_plural = "09. 用户 - 多对多平台记录表"
        unique_together = ['user', 'platform']
        # managed = False   # 是否对数据库表的创建、修改或删除操作

    def __str__(self):
        # return f"{self.username}({self.fullname})"
        return f"{self.user.username}"


# class ContactBook(models.Model):
#     """ 10、*user_contact_book - 用户联系簿表 [1-N] """
#     user = models.ForeignKey(BaseInfo, verbose_name='用户', related_name="user_set", db_constraint=False,
#                              on_delete=models.DO_NOTHING, help_text='')
#     # user_id = models.IntegerField(verbose_name='用户', help_text='')
#     friend = models.ForeignKey(BaseInfo, verbose_name='朋友', related_name="friend_set", db_constraint=False,
#                                on_delete=models.DO_NOTHING, help_text='待删')
#     phone = models.JSONField(verbose_name='手机号', blank=True, null=True, help_text='')
#     phones = models.JSONField(verbose_name='多个手机号', blank=True, null=True, help_text='')
#     telephone = models.JSONField(verbose_name='电话号码', blank=True, null=True, help_text='')
#     telephones = models.JSONField(verbose_name='多个电话号码', blank=True, null=True, help_text='')
#     email = models.EmailField(verbose_name='邮箱', unique=True, blank=True, null=True, db_index=True, help_text='邮箱')
#     qq = models.CharField(verbose_name='QQ', max_length=255, blank=True, null=True, help_text='')
#     address = models.CharField(verbose_name='地址', max_length=255, blank=True, null=True, help_text='')
#     more = models.JSONField(verbose_name='更多', blank=True, null=True, help_text='')
#     remarks = models.CharField(verbose_name='备注', max_length=255, blank=True, null=True, help_text='')
#
#     class Meta:
#         db_table = prefix + 'user_contact_book'
#         verbose_name_plural = "10. 用户 - 联系簿表"
#
#     def __str__(self):
#         # return f"{self.username}({self.fullname})"
#         return f"{self.username}"


class UserSsoServe(models.Model):
    """ 11、User_Sso_Serve - 用户单点登录服务表 [NF1] """
    sso_code = models.CharField(verbose_name='单点登录代码', max_length=255, blank=True, null=True, help_text='')
    sso_name = models.CharField(verbose_name='单点登录名', max_length=255, blank=True, null=True, help_text='')
    sso_url = models.CharField(verbose_name='单点登录地址', max_length=255, blank=True, null=True, help_text='')
    sso_appid = models.CharField(verbose_name='AppID', max_length=255, blank=True, null=True, help_text='')
    sso_account = models.ForeignKey(BaseInfo, verbose_name='单点登录绑定账户', db_column='sso_account_id',
                                    db_constraint=False, on_delete=models.DO_NOTHING, help_text='')
    description = models.CharField(verbose_name='描述', max_length=255, blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_sso_serve'
        verbose_name_plural = "11. 用户 - 单点登录服务表"

    def __str__(self):
        # return f"{self.username}({self.fullname})"
        return f"{self.sso_code}"


class UserSsoToUser(models.Model):
    """ 12、User_Sso_To_User - 多对多单点登录记录表 [NF1] """
    sso_serve = models.ForeignKey(UserSsoServe, verbose_name='单点登录服务ID', db_constraint=False,
                                  on_delete=models.DO_NOTHING, help_text='')
    user = models.ForeignKey(BaseInfo, verbose_name='用户', db_constraint=False, on_delete=models.DO_NOTHING,
                             help_text='')
    sso_unicode = models.CharField(verbose_name='单点登录地址', max_length=255, blank=True, null=True, help_text='？？？')
    sso_ticket = models.CharField(verbose_name='单点登录票据', max_length=255, blank=True, null=True, help_text='')
    union_code = models.CharField(verbose_name='微信唯一ID', max_length=255, blank=True, null=True, help_text='')
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)
    is_delete = models.BooleanField(verbose_name='是否删除', default=False)

    class Meta:
        db_table = prefix + 'user_sso_to_user'
        verbose_name_plural = "12. 用户 - 多对多单点登录记录表"

    def __str__(self):
        return f"{self.sso_unicode}"


# class UserRelateType(models.Model):
#     """ 13、User_Relate_Type - 用户关系类型表 """
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     relate_key = models.CharField(verbose_name='关系搜索key', max_length=32, blank=True, null=True, default="",
#                                   help_text='')
#     relate_name = models.CharField(verbose_name='关系搜索名称', max_length=32, blank=True, null=True, default="",
#                                    help_text='')
#     is_multipeople = models.BooleanField(verbose_name="是否多人", default=0)
#     description = models.CharField(verbose_name='关系描述', max_length=255, blank=True, null=True, default="",
#                                    help_text='')
#
#     class Meta:
#         db_table = prefix + 'user_relate_type'
#         verbose_name_plural = "13. 用户 - 关系类型表"
#
#     def __str__(self):
#         return f"{self.relate_key}"


# class UserRelateToUser(models.Model):
#     """ 14、User_Relate_To_User - 多对多关系类型表 """
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     user = models.ForeignKey(BaseInfo, verbose_name='关系人', blank=True, null=True, db_constraint=False, help_text='',
#                              on_delete=models.DO_NOTHING)
#     with_user = models.ForeignKey(BaseInfo, verbose_name='关联关系人', related_name="relate_with_user_id_alias",
#                                   db_constraint=False, blank=True, null=True, help_text='', on_delete=models.DO_NOTHING)
#     user_relate_type = models.ForeignKey(to=UserRelateType, verbose_name='关系类型ID', blank=True, null=True,
#                                          db_constraint=False, on_delete=models.DO_NOTHING, help_text='', )
#     created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True, null=True, )
#
#     class Meta:
#         db_table = prefix + 'user_relate_to_user'
#         verbose_name_plural = "14. 用户 - 多对多关系类型表"
#
#     def __str__(self):
#         return f"{self.id}-{self.with_user_id}"


class UserBankCards(models.Model):
    id = models.AutoField(verbose_name='ID', primary_key=True)
    user = models.ForeignKey(to="BaseInfo", verbose_name='开户人', blank=True, null=True, help_text='',
                             db_constraint=False, on_delete=models.DO_NOTHING)
    bank_card_num = models.CharField(verbose_name='银行卡号', max_length=32, blank=True, null=True, help_text='', )
    open_account_bank = models.CharField(verbose_name='开户银行', max_length=32, blank=True, null=True, help_text='')
    opening_branch = models.CharField(verbose_name='开户支行', max_length=128, blank=True, null=True, help_text='')
    is_default = models.BooleanField(verbose_name='是否默认卡', blank=True, null=True, help_text='')
    ext = models.JSONField(verbose_name='扩展数据', blank=True, null=True, help_text='')
    sort = models.IntegerField(verbose_name='排序', blank=True, null=True, help_text='')
    remark = models.CharField(verbose_name='备注', max_length=128, blank=True, null=True, help_text='')

    class Meta:
        db_table = prefix + 'user_bank_cards'
        verbose_name_plural = "15. 用户 - 用户银行卡信息"

    def __str__(self):
        return f"{self.bank_card_num}"


# class UserDevice(models.Model):
#     class Meta:
#         db_table = prefix + 'user_device'
#         verbose_name_plural = "16. 用户 - 设备信息"
#
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     user = models.ForeignKey(to="BaseInfo", verbose_name='用户', blank=True, null=True, help_text='', db_constraint=False,
#                              on_delete=models.DO_NOTHING)
#     client_id = models.CharField(verbose_name='推送ID', max_length=50, blank=True, null=True, help_text='', )
#     system = models.CharField(verbose_name='设备系统', max_length=20, blank=True, null=True, help_text='')
#     ip = models.CharField(verbose_name='IP', max_length=255, blank=True, null=True, help_text='')
#     device_model = models.CharField(verbose_name='设备型号', max_length=255, blank=True, null=True, help_text='')
#     update_time = models.DateTimeField(verbose_name='更新时间', blank=True, null=True, help_text='')
#     create_time = models.DateTimeField(verbose_name='创建时间', blank=True, null=True, help_text='')
#
#     def __str__(self):
#         return f"{self.user_id}"


# class UserApplyType(models.Model):
#     class Meta:
#         db_table = prefix + 'user_apply_type'
#         verbose_name_plural = "17. 用户 - 审批类型"
#
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     value = models.CharField(verbose_name='推送ID', max_length=50, blank=True, null=True, help_text='', )
#     type_name = models.CharField(verbose_name='审批备注', max_length=500, blank=True, null=True, help_text='')
#     description = models.CharField(verbose_name='拒绝理由', max_length=500, blank=True, null=True, help_text='')
#     config = models.JSONField(verbose_name='快照', blank=True, null=True, help_text='')
#
#     def __str__(self):
#         return f"{self.id}"


# class UserApplyRecord(models.Model):
#     class Meta:
#         db_table = prefix + 'user_apply_record'
#         verbose_name_plural = "18. 用户 - 审批记录"
#
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     apply_type = models.ForeignKey(to="UserApplyType", verbose_name='审批类型', blank=True, null=True, help_text='',
#                                    on_delete=models.DO_NOTHING)
#     user = models.ForeignKey(BaseInfo, verbose_name='发起审批用户', blank=True, null=True, db_constraint=False,
#                              on_delete=models.DO_NOTHING)
#     verify_user = models.ForeignKey(BaseInfo, verbose_name='审批用户', on_delete=models.DO_NOTHING,
#                                     related_name="verify_user_info", db_constraint=False, blank=True, null=True)
#     result = models.CharField(verbose_name='审批结果', max_length=255, blank=True, null=True, help_text='')
#     remark = models.CharField(verbose_name='审批备注', max_length=500, blank=True, null=True, help_text='')
#     reject_reason = models.CharField(verbose_name='拒绝理由', max_length=500, blank=True, null=True, default="",
#                                      help_text='')
#     snapshot = models.JSONField(verbose_name='快照', blank=True, null=True, default=dict, help_text='')
#     verify_files = models.JSONField(verbose_name='审核文件集', blank=True, null=True, default=dict, help_text='')
#     verify_time = models.DateTimeField(verbose_name='审批时间', blank=True, null=True, help_text='')
#     updated_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True, help_text='')
#     created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True, null=True, help_text='')
#
#     def __str__(self):
#         return f""

# class UserSubCompany(models.Model):
#     class Meta:
#         db_table = prefix + 'user_subcompany'
#         verbose_name_plural = "19. 用户 - 分子公司"
#
#     id = models.AutoField(verbose_name='ID', primary_key=True)
#     name = models.CharField(verbose_name='名称', max_length=100, blank=True, null=True, help_text='', )
#     sort = models.IntegerField(verbose_name='排序', blank=True, null=True, help_text='')
#     created_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True, null=True, help_text='')
#     updated_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True, help_text='')
#
#     def __str__(self):
#         return f"{self.id}"



# encoding: utf-8
"""
@project: djangoModel->custom_merge
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户自定义 字典合并
@created_time: 2022/8/16 15:40
"""


class JJoinList:
    @staticmethod
    def join(l_list: list, r_list: list, l_key: str = "id", r_key: str = "id"):
        join_list = l_list.copy()
        r_map = {str(it.get(r_key)): it for it in r_list}
        for item in join_list:
            if str(item[l_key]) in r_map.keys():
                item.update(r_map[str(item[l_key])])
        return join_list



# encoding: utf-8
"""
@project: djangoModel->user_relate_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户关系服务
@created_time: 2022/12/13 16:45
"""
from decimal import Decimal
import sys

from django.core.paginator import Paginator, EmptyPage
from django.db.models import F, Q, Case, When, Value, CharField, OuterRef
# TODO 已经用了DJango的ORM，就不要用第三方的ORM了 20241206 by Sieyoo
# from orator import DatabaseManager

# from config.config import JConfig

# from tz_sys_biaoxing.utils.j_recur import JRecur
if 'xj_role' in sys.modules:
    from xj_role.services.role_service import RoleService
from ..models import BaseInfo, DetailInfo
# from ..models import UserRelateType, UserRelateToUser
from ..services.user_detail_info_service import DetailInfoService
# 用户关系类型服务
from ..utils.custom_tool import format_params_handle, force_transform_type, filter_fields_handler, dynamic_load_class
from ..utils.utility_method import generate_query_string, parse_integers

# TODO 已经用了DJango的ORM，就不要用第三方的ORM了 20241206 by Sieyoo
# config = JConfig()
# db_config = {
#     config.get('main', 'driver', "mysql"): {
#         'driver': config.get('main', 'driver', "mysql"),
#         'host': config.get('main', 'mysql_host', "127.0.0.1"),
#         'database': config.get('main', 'mysql_database', ""),
#         'user': config.get('main', 'mysql_user', "root"),
#         'password': config.get('main', 'mysql_password', "123456"),
#         "port": config.getint('main', 'mysql_port', "3306")
#     }
# }
# db = DatabaseManager(db_config)


class UserRelateTypeService():
    @staticmethod
    def list(params=None):
        if params is None:
            params = {}
        size = params.get("size", 10)
        page = params.get("page", 20)
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "relate_key", "relate_name", ]
        )
        relate_obj = UserRelateType.objects.filter(**filter_params).values()
        count = relate_obj.count()
        page_set = Paginator(relate_obj, size).get_page(page)
        return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["relate_key", "relate_name", "description", "is_multipeople"]
        )
        try:
            relate_obj = UserRelateType.objects.create(**filter_params)
            return {"id": relate_obj.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, update_params=None):
        if update_params is None:
            update_params = {}
        filter_params = format_params_handle(
            param_dict=update_params,
            filter_filed_list=["relate_key", "relate_name", "description", "is_multipeople"]
        )
        if not pk or not filter_params:
            return None, "没有可修改的数据"
        try:
            relate_obj = UserRelateType.objects.filter(id=pk)
            if not relate_obj:
                return None, "没有可修改的数据"
            relate_obj.update(**filter_params)

            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_obj = UserRelateType.objects.filter(id=pk)
        if not relate_obj:
            return None, None
        try:
            relate_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None


# 用户关系映射服务
class UserRelateToUserService():
    @staticmethod
    def list(params=None, filter_fields=None, only_first=False, **kwargs):
        """
        查询用户关系映射
        :param params: 参数
        :param filter_fields: 过滤字段
        :param only_first: 仅仅查询第一条
        """
        # ------------------------- section 参数处理 start ------------------------------------
        params, err = force_transform_type(variable=params, var_type="dict", default={})
        size, err = force_transform_type(variable=params.get("size"), var_type="int", default=10)
        page, err = force_transform_type(variable=params.get("page"), var_type="int", default=1)
        need_pagination, err = force_transform_type(variable=params.get("need_pagination"), var_type="bool",
                                                    default=True)
        sort = params.get("sort")
        sort = sort if sort and sort in ["created_time", "-created_time", "id", "-id"] else "-created_time"
        default_field_list = [
            "user_id", "with_user_id", "user_relate_type_id", "relate_key", "relate_type_name",
            "username", "fullname", "nickname", "with_username", "with_fullname", "with_nickname",
            "created_time"
        ]
        filter_fields = filter_fields_handler(
            input_field_expression=filter_fields,
            default_field_list=default_field_list,
            all_field_list=default_field_list + ["user_phone", "with_user_phone"]
        )

        # 真名查询
        if params.get("real_name"):
            user_id_list = list(
                DetailInfo.objects.filter(real_name__contains=params.get("real_name")).values("user_id"))
            params.setdefault("user_id_list", [i["user_id"] for i in user_id_list])

        if params.get("with_real_name"):
            user_id_list = list(
                DetailInfo.objects.filter(real_name__contains=params.get("with_real_name")).values("user_id"))
            params.setdefault("with_user_id_list", [i["user_id"] for i in user_id_list])

        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "user_id|int", "user_id_list", "with_user_id_list", "with_user_id|int", "user_relate_type_id|int",
                "relate_key",
                "username", "fullname", "user_phone", "with_user_phone", "nickname", "with_username",
                "with_fullname", "with_nickname",
                "created_time_start", "created_time_end"
            ],
            alias_dict={
                "user_id_list": "user_id__in", "with_user_id_list": "with_user_id__in",
                "created_time_start": "created_time__gte", "created_time_end": "created_time__lte"
            },
            split_list=["user_id_list", "with_user_id_list"]
        )
        # ------------------------- section 参数处理 end   ------------------------------------

        # ------------------------- section 构建ORM start ------------------------------------
        contains_keys = ["user_phone", "with_username", "fullname"]
        contains_params = {}
        contains_query = Q()
        for key in contains_keys:
            if key in filter_params:
                contains_params[key] = filter_params.pop(key)
                contains_query |= Q(**{key + '__icontains': contains_params[key]})
        # 去掉 filter_params 中包含的参数
        filter_params = {key: value for key, value in filter_params.items() if key not in contains_keys}
        relate_user_obj = UserRelateToUser.objects.extra(
            select={"created_time": 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).annotate(
            relate_key=F("user_relate_type__relate_key"),
            relate_type_name=F("user_relate_type__relate_name"),
            username=F("user__username"),
            user_phone=F("user__phone"),
            fullname=F("user__fullname"),
            nickname=F("user__nickname"),
            with_username=F("with_user__username"),
            with_user_phone=F("with_user__phone"),
            with_fullname=F("with_user__fullname"),
            with_nickname=F("with_user__nickname"),
        ).filter(user__is_delete=0, with_user__is_delete=0).order_by(sort).filter(contains_query).filter(
            **filter_params)
        # ------------------------- section 构建ORM end ------------------------------------

        # # ------------------------- section 追加字段 start ------------------------------------
        # # TODO 追加字段
        relate_user_obj = relate_user_obj.values(*filter_fields)
        # if "invite" in filter_params.get("relate_key", []):
        #     for obj in relate_user_obj:
        #         print(obj["user_id"])
        #         ancestor = UserRelateToUser.objects.filter(user__is_delete=0, with_user__is_delete=0,
        #                                                    with_user_id=obj["user_id"]).first()
        #         if ancestor:
        #             username = ancestor.with_user
        #             obj['ancestor_username'] = username
        # relate_user_obj_new.append(company)

        # ------------------------- section 追加字段 end ------------------------------------

        # ------------------------- section 构建返回体 start ------------------------------------
        # 单条返回
        if only_first:
            return relate_user_obj.first(), None

        # 列表返回
        total = relate_user_obj.count()
        if not need_pagination and total <= 200:
            return list(relate_user_obj), None

        # 分页返回
        else:
            try:
                page_set = Paginator(relate_user_obj, size).page(page)
            except EmptyPage:
                return {'total': total, "page": page, "size": size, "list": []}, None

            page_list = list(page_set.object_list)
            # 被邀请人的详细信息
            user_detail_list, err = DetailInfoService.get_list_detail(
                user_id_list=[i["user_id"] for i in page_list],
                filter_fields=["user_id", "avatar", "real_name", "nickname"]
            )
            user_detail_list = user_detail_list if user_detail_list else []
            user_detail_map = {i["user_id"]: i for i in user_detail_list}
            # 邀请人的详细信息
            with_user_detail_list, err = DetailInfoService.get_list_detail(
                user_id_list=[i["with_user_id"] for i in page_list],
                filter_fields=["user_id", "avatar", "real_name", "nickname"]
            )
            with_user_detail_list = with_user_detail_list if with_user_detail_list else []
            with_user_detail_map = {i["user_id"]: i for i in with_user_detail_list}
            # 详细信息拼接
            for j in page_list:
                j["avatar"] = user_detail_map.get(j["user_id"], {}).get("avatar", [])
                j["real_name"] = user_detail_map.get(j["user_id"], {}).get("real_name", "")
                j["nickname"] = user_detail_map.get(j["user_id"], {}).get("nickname", "")

                j["with_avatar"] = with_user_detail_map.get(j["with_user_id"], {}).get("avatar", [])
                j["with_real_name"] = with_user_detail_map.get(j["with_user_id"], {}).get("real_name", "")
                j["with_nickname"] = with_user_detail_map.get(j["with_user_id"], {}).get("nickname", "")
                if "invite" in filter_params.get("relate_key", []):
                    #  判断该用户是否是业务人员
                    role_info, err = RoleService.get_role_list(params={"user_id": j["user_id"]}, only_first=True)
                    is_salesperson = False
                    j['sale_username'] = ""
                    if role_info and role_info.get("role_key") == "BID-SALESMAN":
                        is_salesperson = True
                    if is_salesperson:
                        ancestor = UserRelateToUser.objects.filter(user__is_delete=0, with_user__is_delete=0,
                                                                   with_user_id=j["user_id"]).first()
                        if ancestor:
                            j['sale_username'] = ancestor.with_user.username if ancestor.with_user.username else ""

            return {'total': total, "page": page, "size": size, "list": list(page_set.object_list)}, None
        # ------------------------- section 构建返回体 end   ------------------------------------

    @staticmethod
    def add(params: dict = None, **kwargs):
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)

        # ------------------- section 获取关系类型 start ---------------------------
        relate_key = params.get("user_relate_type_value", params.get("relate_key"))
        if relate_key:
            user_relate_type = UserRelateType.objects.filter(relate_key=relate_key).values().first()
        else:
            user_relate_type = UserRelateType.objects.filter(id=params.get("user_relate_type_id")).values().first()
        if not user_relate_type:
            return None, "不是有效的关系类型"
        params.setdefault("user_relate_type_id", user_relate_type.get("id"))
        # ------------------- section 获取关系类型 end   ---------------------------

        # ------------------- section 过滤字段,并校验合法性 start ---------------------------
        # 参数处理
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type",
                               "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if filter_params.get("user_id", None) is None or filter_params.get("with_user_id",
                                                                           None) is None or filter_params.get(
            "user_relate_type_id", None) is None:
            return None, "参数错误"
        # ------------------- section 过滤字段,并校验合法性 end   ---------------------------

        # ------------------- section 判断是否可以重复绑定 start ---------------------------
        if user_relate_type.get("is_multipeople"):
            # 检查是否已经绑定过
            relate_user_obj = UserRelateToUser.objects.filter(
                user_id=filter_params['user_id'],
                with_user=filter_params['with_user_id'],
                user_relate_type_id=filter_params['user_relate_type_id']
            ).first()
        else:
            # 可多人绑定的用户关系
            relate_user_obj = UserRelateToUser.objects.filter(
                user_id=filter_params['user_id'],
                user_relate_type_id=filter_params['user_relate_type_id']
            ).first()
        if relate_user_obj:
            return None, "无法重复绑定，或者该关系类型不是多人绑定类型。"
        # ------------------- section 判断是否可以重复绑定 end   ---------------------------

        # ------------------- section IO操作 start ---------------------------
        try:
            relate_user_obj = UserRelateToUser.objects.create(**filter_params)
            return {"id": relate_user_obj.id}, None
        except Exception as e:
            return None, str(e)
        # ------------------- section IO操作 end   ---------------------------

    @staticmethod
    def edit(pk=None, params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type",
                               "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if not pk or not params:
            return None, "没有可修改的数据"

        try:
            relate_user_obj = UserRelateToUser.objects.filter(id=pk)
            if not relate_user_obj:
                return None, "没有可修改的数据"
            relate_user_obj.update(**filter_params)
            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_user_obj = UserRelateToUser.objects.filter(id=pk)
        if not relate_user_obj:
            return None, None
        try:
            relate_user_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def distribution_relate(*args, params: dict = None, **kwargs):
        """
        查詢三級分銷接口
        :param params: 参数
        """
        # ------------------------- section 参数处理 start ------------------------------------
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        params.update(kwargs)

        with_user_id, err = force_transform_type(variable=params.get("with_user_id"), var_type="int")
        relate_key, err = force_transform_type(variable=params.get("relate_key"), var_type="str", default="invite")
        select_level, err = force_transform_type(variable=params.get("select_level"), var_type="str")

        select_level = select_level if select_level in ["all", "first", "second", "third"] else "all"
        if not with_user_id:
            return None, "不是一个有效的 邀请人ID"

        size, err = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        page, err = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)

        if params.get("real_name"):
            user_id_list = list(
                DetailInfo.objects.filter(real_name__contains=params.get("real_name")).values("user_id"))
            params.setdefault("user_id_list", [i["user_id"] for i in user_id_list])

        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "relate_key", "relate_type_name", "username", "user_phone", "fullname", "nickname",
                "with_username", "with_user_phone", "with_fullname", "with_nickname", "user_id_list"
            ],
            alias_dict={"user_id_list": "user_id__in"}
        )
        # ------------------------- section 参数处理 end   ------------------------------------

        # ------------------------- section 构建ORM start ------------------------------------
        # 找出第一級別用戶
        first_users = list(UserRelateToUser.objects.filter(
            with_user_id=with_user_id,
            user_relate_type__relate_key="invite"
        ).values("user_id").distinct().order_by("user_id"))
        first_users = [i["user_id"] for i in first_users]

        # 找出第二級別用戶
        second_users = list(UserRelateToUser.objects.filter(
            with_user_id__in=first_users,
            user_relate_type__relate_key=relate_key
        ).values("user_id").distinct().order_by("user_id"))
        second_users = [i["user_id"] for i in second_users]

        # 查詢
        relate_user_obj = UserRelateToUser.objects.extra(
            select={"created_time": 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).annotate(
            relate_key=F("user_relate_type__relate_key"),
            relate_type_name=F("user_relate_type__relate_name"),
            username=F("user__username"),
            user_phone=F("user__phone"),
            fullname=F("user__fullname"),
            nickname=F("user__nickname"),
            with_username=F("with_user__username"),
            with_user_phone=F("with_user__phone"),
            with_fullname=F("with_user__fullname"),
            with_nickname=F("with_user__nickname")
        ).filter(user_relate_type__relate_key=relate_key).filter(**params)

        # 分級別查詢
        if select_level == "first":
            relate_user_obj = relate_user_obj.filter(with_user_id=with_user_id)
        elif select_level == "second":
            relate_user_obj = relate_user_obj.filter(with_user_id__in=first_users)
        elif select_level == "third":
            relate_user_obj = relate_user_obj.filter(with_user_id__in=second_users)
        else:
            relate_user_obj = relate_user_obj.filter(with_user_id__in=second_users + first_users + [with_user_id])

        total = relate_user_obj.count()
        relate_user_list = list(relate_user_obj.values(
            "relate_key",
            "relate_type_name",
            "username",
            "user_phone",
            "fullname",
            "nickname",
            "with_username",
            "with_user_phone",
            "with_fullname",
            "with_nickname",
            "with_user_id",
            "user_id"
        ))
        # ------------------------- section 构建ORM end ------------------------------------

        # ------------------------- section 分页查询 start ------------------------------------
        try:
            page_list = Paginator(relate_user_list, size).page(page)
            page_list = page_list.object_list
        except EmptyPage as e:
            return {"page": page, "size": size, "total": total, "list": []}, None
        # ------------------------- section 分页查询 end   ------------------------------------

        # ------------------------- section 获取用戶信息 start ------------------------------------
        # 被邀请人的详细信息
        user_detail_list, err = DetailInfoService.get_list_detail(
            user_id_list=[i["user_id"] for i in page_list],
            filter_fields=["user_id", "avatar", "real_name", "nickname"]
        )
        user_detail_list = user_detail_list if user_detail_list else []
        user_detail_map = {i["user_id"]: i for i in user_detail_list}
        # 邀请人的详细信息
        with_user_detail_list, err = DetailInfoService.get_list_detail(
            user_id_list=[i["with_user_id"] for i in page_list],
            filter_fields=["user_id", "avatar", "real_name", "nickname"]
        )
        with_user_detail_list = with_user_detail_list if with_user_detail_list else []
        with_user_detail_map = {i["user_id"]: i for i in with_user_detail_list}
        # 详细信息拼接
        for j in page_list:
            j["avatar"] = user_detail_map.get(j["user_id"], {}).get("avatar", [])
            j["real_name"] = user_detail_map.get(j["user_id"], {}).get("real_name", "")
            j["nickname"] = user_detail_map.get(j["user_id"], {}).get("nickname", "")

            j["with_avatar"] = with_user_detail_map.get(j["user_id"], {}).get("avatar", [])
            j["with_real_name"] = with_user_detail_map.get(j["user_id"], {}).get("real_name", "")
            j["with_nickname"] = with_user_detail_map.get(j["user_id"], {}).get("nickname", "")

            # 判断用户分销等级
            if select_level == "first":
                j["level"] = 1
            elif select_level == "second":
                j["level"] = 2
            elif select_level == "third":
                j["level"] = 3
            else:
                if j["user_id"] in first_users:
                    j["level"] = 1
                elif j["user_id"] in second_users:
                    j["level"] = 2
                else:
                    j["level"] = 3
        # ------------------------- section 获取用戶信息 end   ------------------------------------

        # ------------------------- section 項目業務代码 start ------------------------------------
        # 到這裏屬於業務定制代碼，其他項目根據需求修改即可。根據業務進行耦合
        # 查询分销金额
        # TODO 已经用了DJango的ORM，就不要用第三方的ORM了 20241206 by Sieyoo
        data_map = {}
        # data_map = db.table("enroll_enroll"). \
        #     select_raw("""
        #         enroll_enroll.user_id,
        #         ROUND(sum(amount),2) as amount_total,
        #         count(enroll_enroll.id) as count_total
        #     """). \
        #     left_join('user_relate_to_user', 'enroll_enroll.user_id', '=', 'user_relate_to_user.user_id'). \
        #     where_in("enroll_enroll.user_id", [i["user_id"] for i in page_list]). \
        #     where('user_relate_to_user.user_relate_type_id', '=', 1). \
        #     where_in('enroll_enroll.enroll_status_code', [80, 668]). \
        #     where_raw("enroll_enroll.create_time >= user_relate_to_user.created_time"). \
        #     group_by("enroll_enroll.user_id").get()
        # data_map = {i["user_id"]: i for i in data_map}

        #  判断该用户是否是业务人员
        #  如果是业务人员，按照业务人员的分销方案，如果是普通用户则，按照普通用户的营销方案
        role_info, err = RoleService.get_role_list(params={"user_id": with_user_id}, only_first=True)
        is_salesperson = 0
        if role_info and role_info.get("role_key") == "BID-SALESMAN":
            is_salesperson = 1

        def calculate_proportion(amount, select_level, subordinate_user_id):
            amount = str(amount)
            if is_salesperson:  # 是业务员的情况，反三级
                if select_level == 'first':
                    amount = Decimal(amount) * Decimal('0.06')
                elif select_level == 'second':
                    amount = Decimal(amount) * Decimal('0.04')
                elif select_level == 'third':
                    amount = Decimal(amount) * Decimal('0.02')
                else:
                    # 查询全都情况，根据用户ID集合判断，改用的等级
                    if subordinate_user_id in first_users:
                        amount = Decimal(amount) * Decimal('0.06')
                    elif subordinate_user_id in second_users:
                        amount = Decimal(amount) * Decimal('0.04')
                    else:
                        amount = Decimal(amount) * Decimal('0.02')
            else:
                # 不是业务仅仅反一级佣金
                if select_level == "first" or (select_level == "all" and subordinate_user_id in first_users):
                    amount = Decimal(amount) * Decimal('0.04')
                else:
                    amount = 0
            return amount

        for j in page_list:
            j["original_amount"] = float(data_map.get(j["user_id"], {}).get("amount_total", 0))
            j["amount"] = calculate_proportion(
                amount=float(data_map.get(j["user_id"], {}).get("amount_total", 0)),
                select_level=select_level,
                subordinate_user_id=j["user_id"]
            )
            j["order_total"] = float(data_map.get(j["user_id"], {}).get("count_total", 0))
        # ------------------------- section 項目業務代码 end   ------------------------------------

        return {"page": page, "size": size, "total": total, "list": page_list}, None

    # ------------------------------- section 項目業務定制服務 -------------------------------------------
    @staticmethod
    def bind_bxtx_relate(params: dict = None, user_info: dict = None, **kwargs):
        """
        镖行天下绑定用户关系服务
        @note  绑定用户关系 邀请关系和收益关系
        :param params: 请求参数
        :param user_info: 用户信息
        :return: None,err_msg
        """
        user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(user_info)
        params.update(kwargs)
        # 获取绑定用户关系

        # 当前用户ID
        user_id, is_pass = force_transform_type(
            variable=params.get('user_id', params.get('id')),
            var_type="int",
            default=0
        )

        # 邀请人ID
        inviter_id, is_pass = force_transform_type(
            variable=params.get('inviter_id'),
            var_type="int",
            default=0
        )

        # 关系与被关系不能是一个人
        if user_id == inviter_id:
            return None, None

        try:
            # 判断是否是一个有效的用户ID
            inviter = BaseInfo.objects.filter(id=inviter_id).first()
            if not inviter:
                return None, None

            # 绑定邀请人
            data, err = UserRelateToUserService.add({
                "user_id": user_id,
                "with_user_id": inviter_id,
                "user_relate_type_value": "invite"
            })
            if err:
                return None, None

            # 邀请人不存在受益人，如果邀请人是业务，则绑定的受益人也是该邀请人
            res, err = RoleService.is_this_role(user_id=inviter_id, role_key="BID-SALESMAN")  # 如果是业务人员
            if res:
                data, err = UserRelateToUserService.add({
                    "user_id": user_id,
                    "with_user_id": inviter_id,
                    "user_relate_type_value": "beneficiary"
                })
                return None, err

            # 查询邀请人的受益人是谁，如果存在则绑定。
            saler = UserRelateToUser.objects.annotate(relate_key=F("user_relate_type__relate_key")).filter(
                user_id=inviter_id, relate_key="beneficiary"
            ).values().first()
            if saler:
                data, err = UserRelateToUserService.add({
                    "user_id": user_id,
                    "with_user_id": saler.get("with_user_id"),
                    "user_relate_type_value": "beneficiary"
                })
                return None, None

        except Exception as e:
            return None, str(e)
        return None, None

    @staticmethod
    def laowu_bind_bxtx_relate(params: dict = None, user_info: dict = None, is_reset=True, **kwargs):
        """
        （劳务通）绑定用户关系服务
        @note  绑定用户关系 邀请关系和收益关系
        :param params: 请求参数
        :param user_info: 用户信息
        :return: None,err_msg
        """
        user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(user_info)
        params.update(kwargs)
        # 获取绑定用户关系
        user_id_str, is_pass = force_transform_type(variable=params.get('user_id_list'))  # 当前用户ID
        with_user_id, is_pass = force_transform_type(variable=params.get('with_user_id'))  # 邀请人ID
        relate_key, err = force_transform_type(variable=params.get("relate_key"))
        if relate_key:
            user_relate_type = UserRelateType.objects.filter(relate_key=relate_key).values().first()
        else:
            user_relate_type = UserRelateType.objects.filter(id=params.get("user_relate_type_id")).values().first()
        if not user_relate_type:
            return None, "不是有效的关系类型"
        try:
            # 重置原有关系
            if is_reset:
                UserRelateToUser.objects.filter(with_user_id=with_user_id,
                                                user_relate_type_id=user_relate_type.get("id")).delete()
            # 判断是否是一个有效的用户ID
            user_id_list = parse_integers(user_id_str)
            for i in user_id_list:
                # 绑定邀请人
                data, err = UserRelateToUserService.add({
                    "user_id": i,
                    "with_user_id": with_user_id,
                    "user_relate_type_value": relate_key
                })
                if err:
                    return None, str(err)

        except Exception as e:
            return None, str(e)
        return None, None

    @staticmethod
    def bxtx_relate_user(*args, params: dict = None, **kwargs):
        """
        查询用户关系映射
        :param params: 参数
        """
        # ------------------------- section 参数处理 start ------------------------------------
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        params.update(kwargs)

        direct_invite, err = force_transform_type(variable=params.get("direct_invite"), var_type="bool", default=True)
        select_all, err = force_transform_type(variable=params.get("select_all"), var_type="bool", default=False)
        relate_key, err = force_transform_type(variable=params.get("relate_key"))
        if not relate_key:
            relate_key = "invite" if direct_invite else "beneficiary"
            relate_key = "beneficiary" if select_all else relate_key
        user_id, err = force_transform_type(variable=params.get("user_id"), var_type="int")
        if not user_id:
            return None, "不是一个有效的 user_id"
        relate_key, err = force_transform_type(variable=relate_key, var_type="str", default="invite")
        size, err = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        page, err = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        # ------------------------- section 参数处理 end   ------------------------------------

        # ------------------------- section 构建ORM start ------------------------------------
        relate_user_obj = UserRelateToUser.objects.extra(
            select={"created_time": 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).annotate(
            relate_key=F("user_relate_type__relate_key"),
            relate_type_name=F("user_relate_type__relate_name"),
            username=F("user__username"),
            user_phone=F("user__phone"),
            fullname=F("user__fullname"),
            nickname=F("user__nickname"),
            with_username=F("with_user__username"),
            with_user_phone=F("with_user__phone"),
            with_fullname=F("with_user__fullname"),
            with_nickname=F("with_user__nickname")
        ).filter(with_user_id=user_id)

        # if not select_all:
        relate_user_obj = relate_user_obj.filter(user_relate_type__relate_key=relate_key)

        # 如果是查询间接邀请，则获取出直接邀请的用户
        if not direct_invite and not select_all:
            direct_inviter_users = list(UserRelateToUser.objects.filter(
                with_user_id=user_id, user_relate_type__relate_key="invite"
            ).values("user_id").distinct().order_by("user_id"))

            direct_inviter_users = [i["user_id"] for i in direct_inviter_users]
            relate_user_obj = relate_user_obj.exclude(user_id__in=direct_inviter_users)

        total = relate_user_obj.count()
        relate_user_list = list(relate_user_obj.values(
            "relate_key",
            "relate_type_name",
            "username",
            "user_phone",
            "fullname",
            "nickname",
            "with_username",
            "with_user_phone",
            "with_fullname",
            "with_nickname",
            "user_id",
            "with_user_id"
        ))
        # ------------------------- section 构建ORM end ------------------------------------

        # ------------------------- section 分页查询 start ------------------------------------
        try:
            page_list = Paginator(relate_user_list, size).page(page)
            page_list = page_list.object_list
        except EmptyPage as e:
            return {"page": page, "size": size, "total": total, "list": []}, None
        # ------------------------- section 分页查询 end   ------------------------------------
        data_map = {}
        EnrollStatisticsServices, err = dynamic_load_class(import_path="xj_enroll.service.enroll_statistics_services",
                                                           class_name="EnrollStatisticsServices")
        if not err:
            data, err = EnrollStatisticsServices.every_one_total(
                params={"user_id_list": [i["user_id"] for i in page_list]}, need_Pagination=False)
            data_map = {i["user_id"]: i for i in data}

        user_detail_list, err = DetailInfoService.get_list_detail(user_id_list=[i["user_id"] for i in page_list],
                                                                  filter_fields=["user_id", "avatar", "real_name"])
        user_detail_list = user_detail_list if user_detail_list else []
        user_detail_map = {i["user_id"]: i for i in user_detail_list}
        for j in page_list:
            j["amount"] = float(data_map.get(j["user_id"], {}).get("amount_total", 0))
            j["order_total"] = float(data_map.get(j["user_id"], {}).get("count_total", 0))
            j["avatar"] = user_detail_map.get(j["user_id"], {}).get("avatar", [])
            j["real_name"] = user_detail_map.get(j["user_id"], {}).get("real_name", "")

        return {"page": page, "size": size, "total": total, "list": page_list}, None

    # ------------------------------- section 項目業務定制服務 -------------------------------------------

    @staticmethod
    def relate_user_tree(*args, params: dict = None, **kwargs):
        """
        用户关系树状
        劳务系统用 最多两级
        :param params: 参数
        """
        # ------------------------- section 参数处理 start ------------------------------------
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        params.update(kwargs)
        print(params)
        user_id, err = force_transform_type(variable=params.get("user_id"), var_type="int")
        with_user_id, err = force_transform_type(variable=params.get("with_user_id"), var_type="int")
        user_type, err = force_transform_type(variable=params.get("user_type"), var_type="str")
        relate_key, err = force_transform_type(variable=params.get("relate_key"), var_type="str", default="invite")
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "relate_key", "relate_type_name", "username", "user_phone", "fullname", "nickname",
                "with_username", "with_user_phone", "with_fullname", "with_nickname", "user_id_list"
            ],
            alias_dict={"user_id_list": "user_id__in"}
        )
        if user_type == "COMPANY":
            user = BaseInfo.objects.filter(user_type="COMPANY")
            user = user.filter(id=user_id)
            user = user.values("id", "username", "fullname", "nickname")
            if user:
                for i in user:
                    relate_user_obj = UserRelateToUser.objects.extra(
                        select={"created_time": 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
                    ).annotate(
                        relate_key=F("user_relate_type__relate_key"),
                        relate_type_name=F("user_relate_type__relate_name"),
                        username=F("user__username"),
                        user_phone=F("user__phone"),
                        fullname=F("user__fullname"),
                        nickname=F("user__nickname"),
                        with_username=F("with_user__username"),
                        with_user_phone=F("with_user__phone"),
                        with_fullname=F("with_user__fullname"),
                        with_nickname=F("with_user__nickname")
                    ).filter(with_user_id=i['id'], user_relate_type__relate_key=relate_key).values("user_id",
                                                                                                   "username",
                                                                                                   "nickname")
                    i['child'] = list(relate_user_obj)
        else:
            user_relate, err = UserRelateToUserService.bxtx_relate_user(
                params={"user_id": user_id, "relate_key": 'enter'})
            if err:
                return None, err
            user = user_relate['list']

        return list(user), None

    @staticmethod
    def group_tree_user(*args, params: dict = None, **kwargs):
        """
          用户关系树状
          劳务系统用 无限极
          :param params: 参数
        """
        with_user_id, err = force_transform_type(variable=params.get("user_id"), var_type="int")
        relate_key, err = force_transform_type(variable=params.get("relate_key"), var_type="str", default="invite")
        exclude_yourself, err = force_transform_type(variable=params.get("exclude_yourself"), var_type="str",
                                                     default="")
        relate_type = db.table("user_relate_type").where('relate_key', relate_key).first()
        if not relate_type:
            return None, '关系不存在'
        sql = generate_query_string(
            "WITH RECURSIVE cte( user_id, with_user_id, user_relate_type_id, LEVEL, is_parent) AS ( SELECT user_relate_to_user.user_id, user_relate_to_user.with_user_id, user_relate_to_user.user_relate_type_id, 0, 1 FROM user_relate_to_user JOIN user_base_info ON user_relate_to_user.with_user_id = user_base_info.id WHERE user_relate_to_user.with_user_id = ? AND user_relate_to_user.user_relate_type_id = ? UNION ALL SELECT child.user_id, child.with_user_id, child.user_relate_type_id, parent.LEVEL + 1, 0 FROM user_relate_to_user AS child JOIN cte AS parent ON child.with_user_id = parent.user_id JOIN user_base_info ON child.user_id = user_base_info.id WHERE child.user_relate_type_id = ? ) SELECT cte.user_id, cte.with_user_id, cte.LEVEL, cte.user_relate_type_id, parent_user.username AS parent_username, child_user.username AS child_username, parent_user.nickname AS parent_nickname, child_user.nickname AS child_nickname FROM cte LEFT JOIN user_base_info AS parent_user ON cte.with_user_id = parent_user.id LEFT JOIN user_base_info AS child_user ON cte.user_id = child_user.id ORDER BY LEVEL",
            [with_user_id, relate_type.id, relate_type.id])
        results = db.select(sql)
        if results:
            def convert_to_tree(data, parent_key, child_key):
                tree = []
                lookup = {}

                for item in data:
                    item_id = item[parent_key]
                    if item_id not in lookup:
                        node = {
                            'id': item_id,
                            'username': item.get('parent_username') if item_id == with_user_id else item.get(
                                'child_username'),
                            'nickname': item.get('parent_nickname') if item_id == with_user_id else item.get(
                                'child_nickname'),
                            'child': []
                        }
                        lookup[item_id] = node
                        tree.append(node)

                    child_id = item[child_key]
                    if child_id not in lookup:
                        child_node = {
                            'user_id': child_id,
                            'username': item.get('child_username'),
                            'nickname': item.get('child_nickname'),
                            'child': []
                        }
                        lookup[child_id] = child_node
                        lookup[item_id]['child'].append(child_node)

                return tree

            results_list = convert_to_tree(results, 'with_user_id', 'user_id')
        else:
            results = db.table("user_base_info").select("id as user_id", "nickname", "username")\
                .where('id', with_user_id).get()
            results_list = results.items
            for item in results_list:
                item['child'] = []

        if exclude_yourself:
            return [child for child in results_list[0]["child"]], None

        return results_list[0], None



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from base64 import (
    urlsafe_b64encode,
    urlsafe_b64decode,
)
from datetime import datetime, timezone


def b64encode(s: bytes) -> str:
    s_bin = urlsafe_b64encode(s)
    s_bin = s_bin.replace(b'=', b'')
    return s_bin.decode('ascii')


def b64decode(s: str) -> bytes:
    s_bin = s.encode('ascii')
    s_bin += b'=' * (4 - len(s_bin) % 4)
    return urlsafe_b64decode(s_bin)


def uint_b64encode(value: int) -> str:
    length = 1
    rem = value >> 8
    while rem:
        length += 1
        rem >>= 8

    uint_bin = value.to_bytes(length, 'big', signed=False)
    return b64encode(uint_bin)


def uint_b64decode(uint_b64: str) -> int:
    uint_bin = b64decode(uint_b64)

    value = 0
    for b in uint_bin:
        value <<= 8
        value += int(b)
    return value


def get_time_from_int(value: int) -> datetime:
    """
    :param value: seconds since the Epoch
    :return: datetime
    """
    if not isinstance(value, int):  # pragma: no cover
        raise TypeError('an int is required')
    return datetime.fromtimestamp(value, timezone.utc)


def get_int_from_datetime(value: datetime) -> int:
    """
    :param value: datetime with or without timezone, if don't contains timezone
                  it will managed as it is UTC
    :return: Seconds since the Epoch
    """
    if not isinstance(value, datetime):  # pragma: no cover
        raise TypeError('a datetime is required')
    return int(value.timestamp())



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import datetime
from .j_transform_type import JTransformType


class JParameter:
    # 处理字典：白名单、黑名单、别名、移除空值、筛选类型
    @staticmethod
    def format(
        source: dict,
        filter_fields: list = None,
        remove_fields: list = None,
        aliases: dict = None,
        split_fields: list = None,
        split: str = ";",
        is_remove_null: bool = False,
        is_remove_empty: bool = False,
        date_format_dict: dict = None,
        is_validate_type: bool = False,
    ) -> dict:
        """
        字段筛选并替换成别名
        @param source: {dict} 来源参数
        @param filter_fields: {list<key>} 字段白名单。如 ["id|int", "uid"]
        @param remove_fields: {list<key>} 字段黑名单。["id", ...]
        @param aliases: {dict<source_key, target_key>} 字段别名。如 {"id": "user_id"}
        @param split_fields: {list<key>} 指定拆分字段。该参数为了适配使用符号分割成列表。["id_list", ...]
        @param split: {str} 字符串拆分依据。默认分号";"
        @param is_remove_null: {bool} 是否移除值为None字段。默认False
        @param is_remove_empty: {bool} 是否移除值为空值字段。即移除值为 "", 0, "0", None字段；value类型等于False的字段,
        @param date_format_dict: 日期格式化 日期列表，{field：(from_format,to_format),}如：{"field":("%Y-%m-%d %H:%M:%S","%Y-%m-%d %H:%M:%S")}
        @param is_validate_type: 是否验证数据合法性
        @return: param_dict
        """

        result = source.copy()
        # 过滤字段并且得数据类型映射
        new_filter_fields = []  # 拆分字段类型的白名单
        must_type_map = {}  # 从白名单中拆分出字段与类型的映射
        if filter_fields and isinstance(filter_fields, list):
            for ff in filter_fields:
                key_type_pair = ff.split("|", 1)
                [key, key_type] = key_type_pair if len(key_type_pair) == 2 else [key_type_pair[0], None]  # 如果为类型None，则默认是字符串
                must_type_map[key] = key_type
                new_filter_fields.append(key)
            result = {k: v for k, v in result.items() if k in new_filter_fields and (not is_remove_empty or v) and (not is_remove_null or v is not None)}
        # 剔除字段
        if remove_fields and isinstance(remove_fields, list):
            result = {k: v for k, v in result.items() if not k in remove_fields and (not is_remove_empty or v) and (not is_remove_null or v is not None)}
        # 字段拆分
        if split_fields and isinstance(split_fields, list):
            result = {k: (str(v).split(split) if k in split_fields and not isinstance(v, list) else v) for k, v in result.copy().items()}
        # 类型转换
        if must_type_map and isinstance(must_type_map, dict):
            for k, v in result.copy().items():
                v, is_err = JTransformType.to(v, must_type_map.get(k, None))
                if is_err and not is_validate_type:
                    result.pop(k)
                elif is_err and is_validate_type:
                    raise ValueError(k + ":" + is_err)
                else:
                    # 覆盖掉转换的值
                    result[k] = v

        # 日期字段格式转换
        if date_format_dict and isinstance(date_format_dict, dict):
            for k, v in result.copy().items():
                if not date_format_dict.get(k):
                    continue
                try:
                    from_format, to_format = date_format_dict[k]
                    if not isinstance(v, datetime.datetime) and isinstance(v, str):
                        parse_date_obj = datetime.datetime.strptime(v, from_format)
                    else:
                        parse_date_obj = v
                    result[k] = parse_date_obj.strftime(to_format)
                except Exception as e:
                    pass

        # 别名替换
        if aliases and isinstance(aliases, dict):
            result = {aliases.get(k, k): v for k, v in result.copy().items()}

        return result



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import datetime
from parse_json import parse_json


# 强制转换类型数据
def force_transform_type(variable=None, var_type: str = None, default=None, **kwargs):
    """
    强制转换类型，转换规则如下：
    1.变量为空使用默认值返回。认为是空值判断。
    2.转换类型为空则，返回输入变量。认为不需要进行类型转换。
    3.无法转换则返回默认值，转换失败，数据存在问题则饭hi默认值。
    :param default: 数据为空或无法转换的时候返回默认值
    :param variable: 变量
    :param var_type: 转换类型 str|int|bool|float|dict|list|date
    :return: transform_variable, is_err
    """
    if variable is None:  # 为空时候，则返回默认值
        return default, None
    if not var_type:  # 转换类型为空，则不进行转换
        return variable, None
    # 强制类型转换
    try:
        if var_type == "str":
            return variable if isinstance(variable, str) else str(variable), None
        elif var_type == "int":
            return variable if isinstance(variable, int) else int(variable), None
        elif var_type == "bool":
            if variable in (None, '', [], (), {}):
                return None, None
            if variable in (True, False):
                return bool(variable), None
            if variable in ('t', 'True', '1'):
                return True, None
            if variable in ('f', 'False', '0'):
                return False, None
            return None, "'" + str(variable) + "'" + "不是一个有效的布尔类型数据"
        elif var_type == "float":
            return variable if isinstance(variable, float) else float(variable), None
        elif var_type == "only_dict":
            # 不包括列表
            variable = parse_json(variable)
            if isinstance(variable, dict):
                return variable, None
            return default, str(variable) + "无法转换为json格式"
        elif var_type == "dict":
            # 字典包括列表字典
            variable = parse_json(variable)
            if isinstance(variable, dict) or isinstance(variable, list):
                return variable, None
            return default, str(variable) + "无法转换为json格式"
        elif var_type == "list":
            variable = parse_json(variable)
            if isinstance(variable, list):
                return variable, None
            return default, str(variable) + "无法转换为list格式"
        elif var_type == "list_int":
            variable = parse_json(variable)
            result = []
            if isinstance(variable, list):
                for i in variable:
                    # 过滤掉非int类型
                    i, is_pass = force_transform_type(variable=i, var_type="int")
                    if is_pass:
                        continue
                    result.append(i)
                return result, None
            return default, str(variable) + "无法转换为list格式"
        elif var_type == "date":
            format_map = {
                '%Y-%m-%d': "%Y-%m-%d 00:00:00",
                '%Y-%m-%d %H': '%Y-%m-%d %H:00:00',
                '%Y-%m-%d %H:%M': '%Y-%m-%d %H:%M:00',
                '%Y-%m-%d %H:%M:%S': '%Y-%m-%d %H:%M:%S'
            }
            for format_input, format_output in format_map.items():
                try:
                    return datetime.datetime.strptime(variable, format_input).strftime(format_output), None
                except ValueError:
                    pass
            raise ValueError
        else:  # 如果指派的类型不存在，默认值非空则返回默认值，否则原样返回
            return variable if default is None else default, str(var_type) + "不是一个有效的转换类型"
    except TypeError:
        return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"
    except ValueError:
        return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"



# encoding: utf-8
"""
@project: djangoModel->user_apply_services
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户审批服务
@created_time: 2023/8/30 17:09
"""
import datetime
from collections import defaultdict

from django.core.paginator import Paginator, EmptyPage
from django.db.models import F, Count

from ..models import BaseInfo
# from ..models import UserApplyType, UserApplyRecord
from ..services.user_service import UserService
from ..services.user_detail_info_service import DetailInfoService
from ..utils.custom_tool import format_params_handle, force_transform_type

"""
设计思路：
所有的审核都有四个状态
{
"field": "real_name_is_pass", 
"status_map": [{"label": "UN_APPLY", "value": "0"}, {"label": "VERIFY_PASS", "value": "1"}, {"label": "VERIFYING", "value": "2"}, {"label": "VERIFY_REJECT", "value": "3"}]
}

执行步骤：
1. 仅仅在 未申请/审核驳回 状态可以添加申请。
2. 通过联动用户状态，用户字段改成通过。
3. 审核不通过，联动用户字段改成不通过。
4. 审核不通过，用户可以继续提交申请，并生成一条新的记录。
5. 审批记录仅仅有三个状态：待审核、通过、审核驳回, 没有提交记录就是未审批。

核心：
整个审批过程中需要保证 用户详情状态与审批记录结果一致。
审核通过的审批，联动了用户详情里面状态，由于 ‘仅仅在 未申请/审核驳回 状态可以添加申请。’ 所以不可以重复申请审批了。达到了状态一致。
用户审批记录与用户主表和用户详细表的审核状态结合。

各端判断：
1. 移动端使用用户详情里面的字段
2. 后台管理端仅仅看审批记录，看已通过的需要在在审批管理里面，选择已通过卡片就可以看到通过审批记录，这些记录代表审核通过的用户。

查询：
审批主表，联表：用户主表、用户详情表。

审批类型：
代表不通的审批类型，且配置了，用户详情表与审批结果的映射。

表结构：
-- 用户状态审批表
CREATE TABLE `user_apply_record` (
  `id` int NOT NULL AUTO_INCREMENT COMMENT '主键ID',
  `user_id` int NOT NULL COMMENT '申请的用户ID',
  `apply_type_id` int NOT NULL COMMENT '审批类型',
  `verify_user_id` int NOT NULL COMMENT '审核人',
  `result` varchar(500) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT 'VERIFYING' COMMENT '审核结果: 通过：PASS；拒绝：REJECT；忽略：IGNORE; 审核中：VERIFYING',
  `remark` varchar(500) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT '' COMMENT '备注',
  `reject_reason` varchar(500) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT '' COMMENT '拒绝理由',
  `snapshot` json DEFAULT NULL COMMENT '用户状态快照',
  `verifyed_time` timestamp NULL DEFAULT NULL COMMENT '审核时间',
  `created_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '创建时间',
  `updated_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb3 COLLATE=utf8_unicode_ci COMMENT='用户状态审批表';
-- 用户申请类型
CREATE TABLE `user_apply_type` (
  `id` int NOT NULL AUTO_INCREMENT,
  `value` varchar(50) CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci DEFAULT NULL COMMENT '类型搜索key',
  `type_name` varchar(50) CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci NOT NULL COMMENT '类型名称',
  `description` varchar(500) NOT NULL COMMENT '描述',
  `config` json DEFAULT NULL COMMENT '配置，联动字段配置',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=2 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='用户申请类型';
"""


class UserApplyServices:
    VERIFYING = "VERIFYING"
    VERIFY_PASS = "VERIFY_PASS"
    VERIFY_REJECT = "VERIFY_REJECT"
    IGNORE = "IGNORE"
    UN_APPLY = None

    # 审核结果中文映射
    result_to_cn = {
        "VERIFYING": "审核中",
        "VERIFY_PASS": "审核通过",
        "VERIFY_REJECT": "审核拒绝"
    }

    # 审批类型配置案例
    apply_type_config_example = {
        "field": "real_name_is_pass",
        "status_map": [  # 规定的映射，后面的数字根据业务而定
            {"label": "UN_APPLY", "value": "0"},
            {"label": "VERIFY_PASS", "value": "1"},
            {"label": "VERIFYING", "value": "2"},
            {"label": "VERIFY_REJECT", "value": "3"}
        ]
    }

    @staticmethod
    def add_apply_record(params: dict = None, **kwargs):
        """
        添加审批记录, 联动用户状态修改成审核中
        :return: None, err
        """
        # 参数融合
        kwargs, err = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, err = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)

        # --------------------- section 参数有效性验证 start ------------------------
        try:
            # 类型验证
            params = format_params_handle(
                param_dict=params,
                is_remove_empty=False,
                filter_filed_list=[
                    "apply_type_id|int", "user_id|int", "verify_user_id|int", "result|str", "remark|str",
                    "reject_reason|str", "snapshot|dict", "created_time|date", "verify_files|dict"
                ]
            )
            # 必填验证
            for i in ["apply_type_id", "user_id"]:
                if not params.get(i):
                    raise ValueError("不是一个有效的 " + i)
            # 默认初审 审核中
            params.setdefault("result", UserApplyServices.VERIFYING)
        except ValueError as e:
            return None, str(e)

        # 获取申请人基础信息
        user_base_info = BaseInfo.objects.filter(id=params.get("user_id", 0), is_delete=0).values().first()
        if not user_base_info:
            return None, "不存在user_id为 " + str(params.get("params") or " ") + "用户"

        # 获取申请类型
        apply_type_info = UserApplyType.objects.filter(id=params.get("apply_type_id")).values().first()
        if not apply_type_info:
            return None, "不是一个有效的审批类型"
        # --------------------- section 参数有效性验证 end  ------------------------

        # --------------------- section 审批配置校验 start ----------------------------
        # 审批配置信息
        apply_type_config = apply_type_info.get("config", {})
        status_map = apply_type_config.get("status_map", {})
        status_field = apply_type_config.get("field")
        status_value_map = {i["label"]: i["value"] for i in status_map if i.get("label") and i.get("value")}
        if not status_field:
            return None, "审批配置错误，无法读取到审批状态字段"
        if not status_value_map.get(UserApplyServices.VERIFYING) or \
                not status_value_map.get(UserApplyServices.VERIFY_PASS) or \
                not status_value_map.get(UserApplyServices.VERIFY_REJECT):
            return None, "审批配置错误，请检查审类型配置"
        # --------------------- section 审批配置校验 end  ----------------------------

        # --------------------- section 用户详情状态核验 start ----------------------------
        # 获取详细信息
        user_detail_info, err = DetailInfoService.get_detail(user_id=params["user_id"])
        if err:
            return None, "获取用户详细信息失败：" + err
        if not user_detail_info:
            return None, "没有user_id为 （" + str(params.get("user_id", "空")) + "） 的用户详细信息"

        # 审批可提交验证
        verify_status = user_detail_info.get(status_field, "null")
        # 如果为空认为我提交过审核，等同于UN_APPLY
        if verify_status == "null":
            return None, "审批配置错误，无法读取到该用户的审批状态，请检查扩展字段配置"
        if verify_status == status_value_map.get(UserApplyServices.VERIFYING):
            return None, "您有正在审核的申请"
        if verify_status == status_value_map.get(UserApplyServices.VERIFY_PASS):
            return None, "您的审批愿意通过，无需再次申请"
        # --------------------- section 用户详情状态核验 end  ----------------------------

        # --------------------- section 审批及联动修改 start ----------------------------
        # 联动修改用户详细信息
        data, err = DetailInfoService.create_or_update_detail(
            params={
                "user_id": params.get("user_id"),
                status_field: status_value_map.get(UserApplyServices.VERIFYING)
            }
        )
        if err:
            return None, "联动用户详情修改异常：" + err

        # 创建申请记录
        try:
            params.setdefault("snapshot", {
                "field": status_field,
                "update_before": verify_status,
                "update_after": status_value_map.get("verifying")
            })
            apply_record = UserApplyRecord(**params)
            apply_record.save()
        except Exception as e:
            return None, str(e)
        # --------------------- section 审批及联动修改 end  ----------------------------

        return None, None

    @staticmethod
    def edit_apply_record(params: dict = None, pk: int = None, **kwargs):
        """
        添加审批记录, 联动用户状态修改成审核中
        :return: None, err
        """
        # ---------------------- section 参数融合,类型校验 start ----------------------
        kwargs, err = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, err = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)
        pk, err = force_transform_type(variable=pk, var_type="int")
        if pk is None:
            return None, "参数错误，不是一个有效的pk"
        # ---------------------- section 参数融合，类型校验 end  ----------------------

        # --------------------- section 参数有效性验证 start  ------------------------
        # 检查修改的数据是否存在
        instance = UserApplyRecord.objects.filter(id=pk)
        record_instance_info = instance.values(
            "id", "apply_type_id", "user_id", "verify_user_id", "result", "remark",
            "reject_reason", "snapshot", "verify_files", "verify_time", "updated_time",
            "created_time",
        ).first()
        if not record_instance_info:
            return None, "数据不存在，无法修改"

        # 参数白名单，类型验证
        try:
            params = format_params_handle(
                param_dict=params,
                is_remove_empty=False,
                filter_filed_list=[
                    "result|str", "remark|str", "reject_reason|str", "verify_files|dict",
                    "snapshot|dict", "created_time|date", "verify_time|date", "verify_user_id|int"
                ]
            )
            if not params.get("verify_user_id"):
                return None, "审批人不能为空"

        except ValueError as e:
            return None, str(e)
        # --------------------- section 参数有效性验证 end  ------------------------

        # --------------------- section 获取配置，校验参数，校验用户状态 start ------------------------
        # 获取申请类型
        apply_type_info = UserApplyType.objects.filter(id=record_instance_info.get("apply_type_id")).values().first()
        if not apply_type_info:
            return None, "不是一个有效的审批类型"

        # 审批配置信息
        apply_type_config = apply_type_info.get("config", {})
        status_map = apply_type_config.get("status_map", {})
        status_field = apply_type_config.get("field")
        status_value_map = {i["label"]: i["value"] for i in status_map if i.get("label") and i.get("value")}
        if not status_field:
            return None, "审批配置错误，无法读取到审批状态字段"

        # 可审批验证
        user_detail_info, err = DetailInfoService.get_detail(user_id=record_instance_info.get("user_id"))
        if err or not user_detail_info:
            return None, "不存在user_id为 " + str(params.get("params") or "-") + "用户"

        verify_status = user_detail_info.get(status_field, "null")
        if verify_status == "null":
            return None, "审批配置错误，无法读取到该用户的审批状态"

        verify_result = params.get("result")
        if verify_result and not (
                verify_result == UserApplyServices.VERIFY_PASS or verify_result == UserApplyServices.VERIFY_REJECT):
            return None, "不是一个有效的审核类型"

        # --------------------- section 获取配置，校验参数，校验用户状态 end  ------------------------

        # --------------------- section 审批修改,以及联动修改 start ------------------------
        # 联动用户详情状态表修改
        if verify_result:
            user_detail_status = status_value_map.get(verify_result)
            # 联动修改用户详细信息
            user_params = {
                "user_id": record_instance_info.get("user_id"),
                status_field: user_detail_status
            }
            user_params.update(**record_instance_info["verify_files"])
            print("user_params", user_params)
            data, err = DetailInfoService.create_or_update_detail(
                params=user_params
            )
            if err:
                return None, "联动用户详情修改异常：" + err
            params.setdefault("verify_time", datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        # 联动用户身份修改
        if record_instance_info.get("apply_type_id") == 2 and params.get("result") == "VERIFY_PASS":  # 镖师审核通过
            data, err = UserService.bind_user_identity(record_instance_info.get("user_id"), 2)
            if err:
                return None, "联动用户身份异常：" + err
        # 编辑可修改参数
        instance.update(**params)
        # --------------------- section 审批修改,以及联动修改 end  ------------------------

        return None, None

    @staticmethod
    def apply_record_list(params: dict = None, **kwargs):
        # 参数融合
        kwargs, err = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, err = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)

        page, err = force_transform_type(variable=params.get("page"), var_type="int", default=1)
        size, err = force_transform_type(variable=params.get("size"), var_type="int", default=10)
        sort = params.get("sort", "-created_time")
        sort = sort if sort in ["verify_time", "updated_time", "created_time", "-verify_time", "-updated_time",
                                "-created_time"] else "-created_time"
        user_real_name = params.get("user_real_name")
        user_id_card = params.get("user_id_card")

        params = format_params_handle(
            param_dict=params,
            is_remove_empty=False,
            filter_filed_list=[
                "result|str", "remark|str", "reject_reason|str", "verify_files|dict",
                "snapshot|dict", "created_time|date", "verify_time|date", "user_id_list|list_int",
                "verify_user_id_list|list_int", "created_time|date",
                "user_phone", "apply_type_id",
                "created_time_start", "created_time_end", "verify_time_start", "verify_time_end"
            ],
            alias_dict={
                "user_id_list": "user_id__in", "verify_user_id_list": "verify_user_id__in",
                "created_time_start": "created_time__gte",
                "created_time_end": "created_time__lte",
                "verify_time_start": "verify_time__gte",
                "verify_time_end": "verify_time__lte",
            }
        )
        if user_real_name:
            params.update({"verify_files__real_name__icontains": user_real_name})
        if user_id_card:
            params.update({"verify_files__id_card__icontains": user_id_card})
        print("params", params)
        record_obj = UserApplyRecord.objects.annotate(
            apply_type_value=F("apply_type__value"),
            apply_type_name=F("apply_type__type_name"),
            user_username=F("user__username"),
            user_nickname=F("user__nickname"),
            user_phone=F("user__phone"),
            user_user_type=F("user__user_type"),
            user_register_time=F("user__register_time"),
            verify_user_username=F("verify_user__username"),
            verify_user_nickname=F("verify_user__nickname"),
            verify_user_phone=F("verify_user__phone"),
            verify_user_user_type=F("verify_user__user_type"),
        ).extra(
            select={
                'verify_time': 'DATE_FORMAT(verify_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
                'updated_time': 'DATE_FORMAT(updated_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
                'created_time': 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
            }
        ).filter(**params).order_by(sort).values(
            "id", "apply_type_id", "user_id", "verify_user_id", "result", "remark",
            "reject_reason", "snapshot", "verify_files", "verify_time", "updated_time", "created_time",
            "apply_type_value", "apply_type_name",
            "user_username", "user_nickname", "user_phone", "user_user_type", "user_register_time",
            "verify_user_username", "verify_user_nickname", "verify_user_phone", "verify_user_user_type",
        )

        total = record_obj.count()
        paginator = Paginator(record_obj, size)

        try:
            page_set = paginator.page(page)
        except EmptyPage:
            return {'size': int(size), 'page': int(page), 'total': total, 'list': []}, None
        page_list = list(page_set.object_list)

        # 联动用户详细信息
        user_detail_list, err = DetailInfoService.get_list_detail(user_id_list=[i["user_id"] for i in page_list],
                                                                  filter_fields=["user_id", "real_name", "id_card"])
        verify_user_detail_list, err = DetailInfoService.get_list_detail(
            user_id_list=[i["verify_user_id"] for i in page_list], filter_fields=["user_id", "real_name"])
        user_detail_map = {i["user_id"]: i for i in user_detail_list}
        verify_user_detail_map = {i["user_id"]: i for i in verify_user_detail_list}
        for i in page_list:
            i.update(user_detail_map.get(i["user_id"], {}))
            # 替换key，添加前缀verify_
            current_user_verify_user_detail_map = verify_user_detail_map.get(i["verify_user_id"], {})
            if current_user_verify_user_detail_map:
                current_user_verify_user_detail_map = {"verify_" + k: v for k, v in
                                                       current_user_verify_user_detail_map.items()}
            i.update(current_user_verify_user_detail_map)

        return {'size': int(size), 'page': int(page), 'total': total, "result_to_cn": UserApplyServices.result_to_cn,
                'list': page_list}, None

    @staticmethod
    def apply_type_list(params: dict = None, **kwargs):
        return list(UserApplyType.objects.values()), None

    @staticmethod
    def apply_record_agg(params: dict = None, **kwargs):
        # 聚合查询统计计数
        result = UserApplyRecord.objects.values('apply_type_id', 'result').annotate(count=Count('id'))
        new_data = defaultdict(dict)
        [new_data[item['apply_type_id']].update({item['result']: item['count']}) for item in list(result)]
        return {'agg': new_data}, None



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import uuid


# 生成随机字符串，2千万次重复两次。
def get_short_id(length=8):
    """
    生成随机字符串，2千万次重复两次。
    :param length:
    :return: 随机字符串
    """
    length = 8 if length > 8 else length
    dictionary = [
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
        "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
        "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
    ]
    random_id = str(uuid.uuid4()).replace("-", '')  # 注意这里需要用uuid4
    buffer = []
    for i in range(0, length):
        start = i * 4
        end = i * 4 + 4
        val = int(random_id[start:end], 16)
        buffer.append(dictionary[val % 62])
    return "".join(buffer)



# encoding: utf-8
"""
@project: djangoModel->user_bank_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户设备信息
@created_time: 2023/6/8 16:20
"""
from django.core.paginator import Paginator, EmptyPage

from xj_user.models import UserDevice
from ..utils.custom_tool import format_params_handle, force_transform_type


class UserDeviceService():
    @staticmethod
    def get_devices(params=None, pk=None, allow_user_list=None, only_first=False, **kwargs):
        allow_user_list = [i for i in allow_user_list if i] if allow_user_list else None
        # 查看详情
        pk, err = force_transform_type(variable=pk, var_type="int")
        if pk:
            return UserDevice.objects.filter(id=pk).values().first(), None
        else:
            # 参数搜索接口
            params, err = force_transform_type(variable=params, var_type="dict", default={})
            kwargs, err = force_transform_type(variable=kwargs, var_type="dict", default={})
            params.update(kwargs)

            # 分页参数
            size, err = force_transform_type(variable=params.detail("size"), var_type="int", default=10)
            page, err = force_transform_type(variable=params.detail("page"), var_type="int", default=1)

            # 过滤字段
            filter_params = format_params_handle(
                param_dict=params,
                filter_filed_list=[
                    "id", "id_list|list_int", "user_id|int", "user_id_list|list_int", "client_id", "system", "ip", "device_model",
                ],
                alias_dict={"id_list": "id__in", "user_id_list": "user_id__in"}
            )

            # 排序字段
            sort = params.detail("sort")
            sort = sort if sort in ["id", "-id", "update_time", "create_time", "-update_time", "-create_time"] else "-id"

            # 构建ORM进行查询
            user_device_set = UserDevice.objects
            if not allow_user_list is None and isinstance(allow_user_list, list):  # 筛选可以访问的列表
                user_device_set = user_device_set.filter(id__in=allow_user_list)
            cards_obj = user_device_set.filter(**filter_params).order_by(sort).values()
            count = cards_obj.count()

            if only_first:
                return cards_obj.first(), None

            try:
                page_set = Paginator(cards_obj, size).get_page(page)
            except EmptyPage:
                return {'count': count, "page": page, "size": size, "list": []}, None

            return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None

    @staticmethod
    def create_or_update(params: dict = None):
        if params is None:
            params = {}
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user_id|int", "client_id", "system", "ip", "device_model"]
        )

        user_id = params.get("user_id")
        if not user_id:
            return None, "参数错误，用户ID必传"

        user_device_obj = UserDevice.objects.filter(user_id=user_id)
        try:
            if not user_device_obj.first():
                device_set = UserDevice.objects.create(**params)
                return {"id": device_set.id}, None
            else:
                user_device_obj.update(**params)
                return {"id": user_id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        device_obj = UserDevice.objects.filter(id=pk)
        if not device_obj:
            return None, None
        try:
            device_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user_id|int", "client_id", "system", "ip", "device_model"]
        )
        try:
            device_set = UserDevice.objects.create(**filter_params)
            return {"id": device_set.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, update_params=None):
        if update_params is None:
            update_params = {}
        filter_params = format_params_handle(
            param_dict=update_params,
            filter_filed_list=["user_id|int", "client_id", "system", "ip", "device_model"]
        )
        if not pk or not filter_params:
            return None, "没有可修改的数据"
        try:
            device_obj = UserDevice.objects.filter(id=pk)
            if not device_obj:
                return None, "没有可修改的数据"
            device_obj.update(**filter_params)

            return None, None
        except Exception as e:
            return None, str(e)
